/*
 *  model.c
 *  LIME, The versatile 3D line modeling tool 
 *
 *  Created by Christian Brinch on 11/05/07.
 *  Copyright 2006-2011, Christian Brinch, 
 *  <brinch@nbi.dk>
 *  Niels Bohr institutet
 *  University of Copenhagen
 *	All rights reserved.
 *
 */

#include "lime.h"

/******************************************************************************/

void
input(inputPars *par, image *img){
/*
 * Basic parameters. See cheat sheet for details.
 */
  par->radius			= 2000*AU;
  par->minScale	   		= 0.5*AU;
  par->pIntensity    	= 800;
  par->sinkPoints    	= 3000;
  par->dust				= "/Users/christianbrinch/LimePackage/trunk/example/jena_thin_e6.tab";
  par->outputfile 		= "populations.pop";
  par->gridfile			= "grid.vtk";
  par->sampling			= 1;
  par->antialias		= 1;

/* 
 * Definitions for image #0. Add blocks for additional images.
 */
  img[0].freq			= 267e9;          // zero-indexed J quantum numbe
  img[0].pxls			= 101;	      // Pixels per dimension
  img[0].imgres			= .1;
  img[0].theta			= 0.0;		  // 0: face-on, pi/2: edge-on
  img[0].distance		= 140*PC;	  // source distance in m
  img[0].source_vel		= 0;          // source velocity in m/s
  img[0].unit			= 0;		  // 0:Kelvin 1:Jansky/pixel 2:SI 3:Lsun/pixel 4:tau
  img[0].filename		= "image0.fits";	// Output filename
}

/******************************************************************************/

void
density(double x, double y, double z, double *density){	
  double r=sqrt((x+500*AU)*(x+5000*AU)+y*y+z*z);
  
    density[0]=1.e12*pow(sqrt(x*x+y*y+z*z)/(1000*AU),-1.5);
  //density[0] = pow(0.2e3,2.)/(2*PI*GRAV*pow(r,2.))/(2.4*AMU);
//  double rhoc=1e-12;
//  density[0]=rhoc/(2.4*AMU);
}

/******************************************************************************/

void
temperature(double x, double y, double z, double *temperature){
  temperature[0]=30;
}

/******************************************************************************/

void
abundance(double x, double y, double z, double *abundance){
}

/******************************************************************************/

void
doppler(double x, double y, double z, double *doppler){
}

/******************************************************************************/

void
velocity(double x, double y, double z, double *vel){
}

/******************************************************************************/

void
magfield(double x, double y, double z, double *B){
	double r, theta, phi;
		
	double Br, Bt, Bf;
	double w;

	r=sqrt(x*x+y*y+z*z);
	theta = acos(z/r);
	phi   = atan2(y,x);

	w = cos(theta);
	
	Br = -2.*w/pow(r,3.);
	Bt = -sqrt(1.-w*w)/pow(r,3.);
	Bf = 0.;	
	
	B[0] = Br*sin(theta)*cos(phi) + Bt*cos(theta)*cos(phi) - Bf*sin(phi);
	B[1] = Br*sin(theta)*sin(phi) + Bt*cos(theta)*sin(phi) + Bf*cos(phi);
	B[2] = Br*cos(theta)          - Bt*sin(theta);
	
}
/******************************************************************/


