/*
 *  curses.c
 *  LIME, The versatile 3D line modeling environment 
 *
 *  Created by Christian Brinch on 29/10/08.
 *  Copyright 2006-2011, Christian Brinch, 
 *  <brinch@nbi.dk>
 *  Niels Bohr institutet
 *  University of Copenhagen
 *	All rights reserved.
 *
 *  Adopted for ARTIST on 21/10/2011
 *  by
 *  Reinhold Schaaf
 *  Argelander-Institut fuer Astronomie
 *  Universitaet Bonn
 *  rschaaf@astro.uni-bonn.de
 *
 *  For further info about copyright and license see included license.txt file
 *
 */

/*#include <curses.h>*/
#include <time.h>

#include "lime.h"

void
clearMessage() {
    memset (status.message, '\0', 200);
}

void
greetings(){
	clearMessage();
	sprintf(status.message,
			"%s\n%s\n",
			"*** LIME, The versatile line modeling engine, Ver.1.03 (r174)",
			"*** Copyright 2006--2011, Christian Brinch <brinch@strw.leidenuniv.nl>");
}

void
screenInfo(){
	/*
	 * Initialize global status variable:
	 * */
    status.progressGridBuilding = 0.;
    status.progressGridSmoothing = 0.;
    status.statusGrid = 0;
    status.numberIterations = 0;
    status.minsnr = 0.;
    status.median = 0.;
    status.progressPhotonPropagation = 0.;
    status.progressRayTracing = 0.;
    status.statusRayTracing = 0;
    status.statusGlobal = 0;
    status.error = 0;

    /*
     * Code for curses output to screen kept for reference
     *
	move(4,6);  printw("Building Delaunay grid:");
	move(4,56); printw("|");
	move(5,6);  printw("Smoothing grid        :");
	move(5,56); printw("|");
	move(7,6);  printw("Statistics            :");
	move(9,6);  printw("Iterations            :"); 
	move(10,6);  printw("Propagating photons   :");
	move(10,56); printw("|");
	move(13,6); printw("Ray-tracing model     :");
	move(13,56);printw("|");	
	refresh();
    */
}

void
done(int line){
	switch (line) {
	case 5:
		status.statusGrid = 1;
		break;
	case 13:
		status.statusRayTracing = 1;
		break;
	}
    /*
     * Code for curses output to screen kept for reference
     *
	move(line,57); printw(" [done]");
    refresh();
    */
}

void
progressbar(double percent, int line){
	switch (line) {
	case 4:
		status.progressGridBuilding = percent;
		break;
	case 5:
		status.progressGridSmoothing = percent;
		break;
	case 10:
		status.progressPhotonPropagation = percent;
		break;
	case 13:
		status.progressRayTracing = percent;
		break;
	}
    /*
     * Code for curses output to screen kept for reference
     *
	int i;
	if((int)(100*percent)%4==0){
		for(i=0;i<(int)(percent*25.);i++){
			move(line,30+i);
			printw("#");
		}
		refresh();
	}	
	*/
}

void 
progressbar2(int prog, double percent, double minsnr, double median){
	status.numberIterations = prog;
    if(percent<100) {
    	status.progressPhotonPropagation = 0.;
    }
	status.minsnr = minsnr;
    status.median = median;

	/*
     * Code for curses output to screen kept for reference
     *
	move(7,30);		 printw("Min(SNR) %3.2f", minsnr);
	move(8,30);		 printw("Median(SNR) %4.2f",median);
	move(9,30+prog); printw("#");
	if(percent<100) {
		move(10,30);	 printw("                         ");
	}
	refresh();	
	*/
}

void 
goodnight(int initime, char *filename){
	int runtime=time(0)-initime;

	status.statusGlobal = 1;

	clearMessage();
	sprintf(status.message,
			"*** Program ended succesfully; runtime: %3dh %2dm %2ds***",
			runtime/3600, runtime/60%60, runtime%60);
}

void
quotemass(double mass){
	clearMessage();
	sprintf(status.message,
			"Total mass contained in model: %3.2e solar masses",
			mass);
}



void 
warning(char message[80]){
	clearMessage();
	sprintf(status.message, "%s", message);
}

void 
bail_out(char message[80]){
	status.error = 1;

	clearMessage();
	sprintf(status.message, "%s", message);
}
