////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////


%define DOCSTRING
"LIME, The versatile 3D line modeling environment
"
%enddef
%module(docstring=DOCSTRING) lime
%{
#include "iLime.h"
%}


%pythoncode %{
__version__ = "1.0.3"
%}

////////////////////////////////////////////////////////////////////////////////
//
// Docstrings:
%feature("docstring")  inputPars "For internal use only; use LimeInputPars instead";

%feature("docstring")  LimeInputPars "LIME's basic settings
		
See the documentation for details";

%feature("docstring")  image "For internal use only; use LimeImage instead";

%feature("docstring")  LimeImage "Information about an image to be created by LIME
		
See the documentation for details";

%feature("docstring") ProgStatus "Status of the LIME run
		
See the documentation for details";

%feature("docstring") getStatus "Returns a ProgStatus object with information about the current status of the LIME run";

%feature("docstring") silent "For internal use only; use setSilent and getSilent instead";

%feature("docstring") setSilent "setSilent(int silent) 
 
Enables or disables feedback about the status of the LIME run via a ProgStatus object. 

Parameters:
-----------
int silent: If 0, do not update the ProgStatus object. Otherwise, update it.";

%feature("docstring") getSilent "int getSilent() 
 
Returns 0 if ProgStatus object is not updated; any other value means that it is updated.";

%feature("docstring") run "For internal use only; use runLime instead";


%include "carrays.i"
%array_class(image, imageArray);

// This tells SWIG to treat char ** as a special case
%typemap(in) char ** {
  /* Check if is a list */
  if (PyList_Check($input)) {
    int size = PyList_Size($input);
    int i = 0;
    $1 = (char **) malloc((size+1)*sizeof(char *));
    for (i = 0; i < size; i++) {
      PyObject *o = PyList_GetItem($input,i);
      if (PyString_Check(o))
	$1[i] = PyString_AsString(PyList_GetItem($input,i));
      else {
	PyErr_SetString(PyExc_TypeError,"list must contain strings");
	free($1);
	return NULL;
      }
    }
    $1[i] = 0;
  } else {
    PyErr_SetString(PyExc_TypeError,"not a list");
    return NULL;
  }
}

// This cleans up the char ** array we malloc'd before the function call
%typemap(freearg) char ** {
  free((char *) $1);
}

// Input parameters 
// Lime internal:
typedef struct {
	double radius,minScale,tcmb;
	int ncell,sinkPoints,pIntensity,nImages,nSpecies,blend;
	char *outputfile, *inputfile;
	char *gridfile;
	char *pregrid;
	char *dust;
	int sampling,collPart,lte_only,antialias,polarization;
	char **moldatfile;
} inputPars;

// Interface (to be used in Python):
typedef struct {
	double radius,minScale,tcmb;
	int sinkPoints,pIntensity,blend;
	char *outputfile;
	char *gridfile;
	char *pregrid;
	char *moldatfile;
	char *dust;
	int sampling,lte_only,antialias,polarization;
} LimeInputPars;



// Image information
// Lime internal:
typedef struct {
	int doline;
	int nchan,trans;
	spec *pixel;	
	double velres;
	double imgres;
	int pxls;
	int unit;
	double freq,bandwidth;
	char *filename;
	double source_vel;
	double theta,phi;
	double distance;
} image;

// Interface (to be used in Python):
typedef struct {
	int nchan,trans;
	double velres;
	double imgres;
	int pxls;
	int unit;
	double freq,bandwidth;
	char *filename;
	double source_vel;
	double theta,phi;
	double distance;
} LimeImage;


// Program status:
typedef struct {
    float progressGridBuilding;
    float progressGridSmoothing;
    int statusGrid;              /* 1 if done */
    int numberIterations;
    double minsnr;
    double median;
    float progressPhotonPropagation;
    float progressRayTracing;
    int statusRayTracing;       /* 1 if done */
    
    int statusGlobal;           /* 1 if done */
    int error;                  /* 1 if Error occured */
    
    char message[200];
} ProgStatus;

ProgStatus getStatus();

// LIME's global variables:
int silent;
void setSilent(int silent);
int getSilent();

// Run Lime:
void run(inputPars *par, image images[], char **moldatfiles);

// Python code to check input to LIME:
%include iPyLime.i

