////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

// Template header file for model classes
// ADJUST mark lines that must be modified

// ADJUST: Replace TEMPLATE with reasonable string
#ifndef MODEL_TEMPLATE_H
#define MODEL_TEMPLATE_H

#include "model.h"

#include <set>

// ADJUST: Replace class name
// ADJUST: Remove interfaces that do not apply
class ModelTemplate : public Model,
public IDensityProvider,		// related with method density
public ITemperatureProvider,	// related with method temeprature
public IAbundanceProvider,		// related with method abundance
public ITdustProvider,			// related with method tdust
public IDopplerProvider,		// related with method doppler
public IVelocityProvider,		// related with method velocity
public IBmagProvider			// related with method bmag
{
  // Model interface:
public:
  // ADJUST: Rename constructor and destructor
  ModelTemplate();
  virtual ~ModelTemplate();

  static string modelID;
  const ModelReg getModelReg();
  const set<string> getParamIDs();
  const ParamReg* getParamReg(string paramID);

protected:
  virtual void onFinalizeConfiguration();

  // IProvider interfaces:
public:
  // ADJUST: Remove methods that do not apply
  // Related with IDensityProvider
  virtual const double
  density(const double x, const double y, const double z);

  // Related with ITemperatureProvider
  virtual const double
  temperature(const double x, const double y, const double z);

  // Related with IAbundanceProvider
  virtual const double
  abundance(const double x, const double y, const double z);

  // Related with ITdustProvider
  virtual const double
  t_dust(const double x, const double y, const double z);

  // Related with IDopplerProvider
  virtual const double
  doppler(const double x, const double y, const double z);

  // Related with IVelocityProvider
  virtual const void
  velocity(const double x, const double y, const double z, double* v);

  // Related with IBmagProvider
  virtual const void
  bmag(const double x, const double y, const double z, double* b);

private:
  // ADJUST: Replace with necessary member variables
  double m_pDbl;
  int m_pInt;
  string m_pStr;
  unsigned m_pEnumIndex;
};

#endif //MODEL_TEMPLATE_H
