
// File: icpp_modellib_8cc.xml
/*  Exploration  */

%feature("docstring")  getModelIDs "* The IDs of all models
registered with the model library* set<string> getModelIDs()

Returns the IDs of all registered models. ";

%feature("docstring")  isRegisteredModel "* Checks if a model is
registered with the model library* bool isRegisteredModel(const string
modelID)

Returns true if model with modelID is registered, else false.

Parameters:
-----------

modelID:  ID of the model. ";

%feature("docstring")  getModelName "* Name of a model* string
getModelName(const string modelID)

Returns the name of the model.

Parameters:
-----------

modelID:  ID of the model.

Exceptions:
-----------

invalid_argument:  if no model with modelID registered. ";

%feature("docstring")  getModelDesc "* Description of a model* string
getModelDesc(const string modelID)

Returns the description of the model.

Parameters:
-----------

modelID:  ID of the model.

Exceptions:
-----------

invalid_argument:  if no model with modelID registered. ";

%feature("docstring")  getModelBibref "* Bibliographic reference of a
model* string getModelBibref(const string modelID)

Returns the bibliographic reference of the model.

Parameters:
-----------

modelID:  ID of the model.

Exceptions:
-----------

invalid_argument:  if no model with modelID registered. ";

%feature("docstring")  getParamIDs "* The IDs of all parameters
registered for a model* set<string> getParamIDs(const string modelID)

Returns the IDs of all registered parameters for model.

Parameters:
-----------

modelID:  ID of the model.

Exceptions:
-----------

invalid_argument:  if no model with modelID registered. ";

%feature("docstring")  isRegisteredParam "* Checks if a model
parameter is registered with the model library* bool
isRegisteredParam(const string modelID, const string paramID)

Returns true if parameter with paramID is registered for model with
modelID, else false.

Parameters:
-----------

modelID:  ID of the model.

paramID:  ID of the parameter. ";

%feature("docstring")  getParamName "* Name of a model parameter*
string getParamName(const string modelID, const string paramID)

Returns the name of the model parameter.

Parameters:
-----------

modelID:  ID of the model.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no model with modelID or parameter with paramID
registered. ";

%feature("docstring")  getParamDesc "* Description of a model
parameter* string getParamDesc(const string modelID, const string
paramID)

Returns the description of the model parameter.

Parameters:
-----------

modelID:  ID of the model.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no model with modelID or parameter with paramID
registered. ";

%feature("docstring")  getParamType "* Datatype of a model parameter*
string getParamType(const string modelID, const string paramID)

Returns the datatype of the model parameter. Valid datatypes are
\"double\", \"int\", and \"string\".

Parameters:
-----------

modelID:  ID of the model.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no model with modelID or parameter with paramID
registered. ";

%feature("docstring")  getParamUnit "* Unit of a model parameter*
string getParamUnit(const string modelID, const string paramID)

Returns the unit of the model parameter.

Parameters:
-----------

modelID:  ID of the model.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no model with modelID or parameter with paramID
registered.

domain_error:  if parameter with paramID is of type without unit. ";

%feature("docstring")  getParamDefValInt "* Default value of an
integer model parameter* int getParamDefValInt(const string modelID,
const string paramID)

Returns the integer default value of the model parameter.

Parameters:
-----------

modelID:  ID of the model.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no model with modelID or parameter with paramID
registered.

domain_error:  if parameter with paramID is not of type int. ";

%feature("docstring")  getParamDefValDouble "* Default value of a
double model parameter* double getParamDefValDouble(const string
modelID, const string paramID)

Returns the double default value of the model parameter.

Parameters:
-----------

modelID:  ID of the model.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no model with modelID or parameter with paramID
registered.

domain_error:  if parameter with paramID is not of type double. ";

%feature("docstring")  getParamDefValString "* Default value of a
string model parameter* string getParamDefValString(const string
modelID, const string paramID)

Returns the string default value of the model parameter.

Parameters:
-----------

modelID:  ID of the model.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no model with modelID or parameter with paramID
registered.

domain_error:  if parameter with paramID is not of type string. ";

%feature("docstring")  getParamEnumValues "* Values of an enum model
parameter* vector<double> getParamEnumValues(const string modelID,
const string paramID)

Returns the enum values of the model parameter.

Parameters:
-----------

modelID:  ID of the model.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no model with modelID or parameter with paramID
registered.

domain_error:  if parameter with paramID is not of type enum. ";

%feature("docstring")  getParamEnumDefIndex "* Default index of an
enum model parameter* unsigned getParamEnumDefIndex(const string
modelID, const string paramID)

Returns the default index of the model parameter.

Parameters:
-----------

modelID:  ID of the model.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no model with modelID or parameter with paramID
registered.

domain_error:  if parameter with paramID is not of type enum. ";

%feature("docstring")  getResultIDs "* The IDs of all results
registered with the model library* set<string> getResultIDs()

Returns the IDs of all registered results. ";

%feature("docstring")  isRegisteredResult "* Checks if a result is
registered with the model library* bool isRegisteredResult(const
string resultID)

Returns true if result with resultID is registered, else false.

Parameters:
-----------

resultID:  ID of the result. ";

%feature("docstring")  getResultName "* Name of a result* string
getResultName(const string resultID)

Returns the name of the result.

Parameters:
-----------

resultID:  ID of the result.

Exceptions:
-----------

invalid_argument:  if no result with resultID registered. ";

%feature("docstring")  getResultDesc "* Description of a result*
string getResultDesc(const string resultID)

Returns the description of the result.

Parameters:
-----------

resultID:  ID of the result.

Exceptions:
-----------

invalid_argument:  if no result with resultID registered. ";

%feature("docstring")  getResultUnit "* Unit of a result* string
getResultUnit(const string resultID)

Returns the unit of the result.

Parameters:
-----------

resultID:  ID of the result.

Exceptions:
-----------

invalid_argument:  if no result with resultID registered. ";

%feature("docstring")  getResultType "* Type of a result* string
getResultType(const string resultID)

Returns the type of the result. Valid types are \"scalar\" and
\"vector\"

Parameters:
-----------

resultID:  ID of the result.

Exceptions:
-----------

invalid_argument:  if no result with resultID registered. ";

%feature("docstring")  getModelResultIDs "* The IDs of all results
registered for a model* set<string> getModelResultIDs(const string
modelID)

Returns the resultIDs registered for the model.

Parameters:
-----------

modelID:  ID of the model.

Exceptions:
-----------

invalid_argument:  if no model with modelID registered. ";

%feature("docstring")  isResultModel "* Checks if a Result is
registered for a Model* bool isResultModel(const string resultID,
const string modelID)

Return True if Result with resultID is registered for the Model with
modelID, else False

Parameters:
-----------

resultID:  ID of the result.

modelID:  ID of the model.

Exceptions:
-----------

invalid_argument:  if no result with resultID registered.

invalid_argument:  if no model with modelID registered. ";

%feature("docstring")  getFunctionIDs "* The IDs of all functions
registered with the model library* set<string> getFunctionIDs()

Returns the IDs of all registered functions. ";

%feature("docstring")  isRegisteredFunction "* Checks if a function
is registered with the model library* bool isRegisteredFunction(const
string functionID)

Returns true if function with functionID is registered, else false.

Parameters:
-----------

functionID:  ID of the model. ";

%feature("docstring")  getFunctionName "* Name of a function* string
getFunctionName(const string functionID)

Returns the name of the function.

Parameters:
-----------

functionID:  ID of the function.

Exceptions:
-----------

invalid_argument:  if no function with functionID registered. ";

%feature("docstring")  getFunctionDesc "* Description of a function*
string getFunctionDesc(const string functionID)

Returns the description of the function.

Parameters:
-----------

functionID:  ID of the function.

Exceptions:
-----------

invalid_argument:  if no function with functionID registered. ";

%feature("docstring")  getFunctionType "* Type of a function* string
getFunctionType(const string functionID)

Returns the type of the function. Valid types are \"scalar\" and
\"vector\".

Parameters:
-----------

functionID:  ID of the function.

Exceptions:
-----------

invalid_argument:  if no function with functionID registered. ";

%feature("docstring")  getFunctionResultIDs "* The IDs of all Results
to which a Function can be linked* set<string>
getFunctionResultIDs(const string functionID)

Returns the resultIDs of all Results to which the Function can be
linked

Parameters:
-----------

functionID:  ID of the function.

Exceptions:
-----------

invalid_argument:  if no function with functionID registered. ";

%feature("docstring")  getResultFunctionIDs "* The IDs of all
Functions that can be linked to a Result* set<string>
getResultFunctionIDs(const string resultID)

Returns the functionIDs of all Functions that can be linked to the
Result.

Parameters:
-----------

resultID:  ID of the result.

Exceptions:
-----------

invalid_argument:  if no result with resultID registered. ";

%feature("docstring")  isResultFunction "* Checks if a Function can
be linked to a Result* bool isResultFunction(const string resultID,
const string functionID)

True if the Function with functionID can be linked to the Result with
resultID, else False

Parameters:
-----------

resultID:  ID of the result.

functionID:  ID of the function. ";

%feature("docstring")  getFunctionParamIDs "* The IDs of all
parameters registered for a function* set<string>
getFunctionParamIDs(const string functionID)

Returns the IDs of all registered parameters for function.

Parameters:
-----------

functionID:  ID of the function.

Exceptions:
-----------

invalid_argument:  if no function with functionID registered. ";

%feature("docstring")  isRegisteredFunctionParam "* Checks if a
function parameter is registered with the model library* bool
isRegisteredFunctionParam(const string functionID, const string
paramID)

Returns true if parameter with paramID is registered for function with
functionID, else false.

Parameters:
-----------

functionID:  ID of the function.

paramID:  ID of the parameter. ";

%feature("docstring")  getFunctionParamName "* Name of a function
parameter* string getFunctionParamName(const string functionID, const
string paramID)

Returns the name of the function parameter.

Parameters:
-----------

functionID:  ID of the function.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no function with functionID or parameter with
paramID registered. ";

%feature("docstring")  getFunctionParamDesc "* Description of a
function parameter* string getFunctionParamDesc(const string
functionID, const string paramID)

Returns the description of the function parameter.

Parameters:
-----------

functionID:  ID of the function.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no function with functionID or parameter with
paramID registered. ";

%feature("docstring")  getFunctionParamType "* Datatype of a function
parameter* string getFunctionParamType(const string functionID, const
string paramID)

Returns the datatype of the function parameter. Valid datatypes are
\"double\", \"int\", and \"string\".

Parameters:
-----------

functionID:  ID of the function.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no function with functionID or parameter with
paramID registered. ";

%feature("docstring")  getFunctionParamDefValInt "* Default value of
an integer function parameter* int getFunctionParamDefValInt(const
string functionID, const string paramID)

Returns the integer default value of the function parameter.

Parameters:
-----------

functionID:  ID of the function.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no function with functionID or parameter with
paramID registered.

domain_error:  if parameter with paramID is not of type int. ";

%feature("docstring")  getFunctionParamDefValDouble "* Default value
of a double function parameter* double
getFunctionParamDefValDouble(const string functionID, const string
paramID)

Returns the double default value of the function parameter.

Parameters:
-----------

functionID:  ID of the function.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no function with functionID or parameter with
paramID registered.

domain_error:  if parameter with paramID is not of type double. ";

%feature("docstring")  getFunctionParamDefValString "* Default value
of a string function parameter* string
getFunctionParamDefValString(const string functionID, const string
paramID)

Returns the string default value of the function parameter.

Parameters:
-----------

functionID:  ID of the function.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no function with functionID or parameter with
paramID registered.

domain_error:  if parameter with paramID is not of type string. ";

%feature("docstring")  getFunctionParamEnumValues "* Values of an
enum function parameter* vector<double>
getFunctionParamEnumValues(const string functionID, const string
paramID)

Returns the enum values of the function parameter.

Parameters:
-----------

functionID:  ID of the function.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no function with functionID or parameter with
paramID registered.

domain_error:  if parameter with paramID is not of type enum. ";

%feature("docstring")  getFunctionParamEnumDefIndex "* Default index
of an enum function parameter* unsigned
getFunctionParamEnumDefIndex(const string functionID, const string
paramID)

Returns the default index of the function parameter.

Parameters:
-----------

functionID:  ID of the function.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no function with functionID or parameter with
paramID registered.

domain_error:  if parameter with paramID is not of type enum. ";

/*  Setup  */

%feature("docstring")  setCurrentModel "* Select a Current Model*
void setCurrentModel(const string modelID)

All Parameters of the selected Model are set to their default values.

Parameters:
-----------

modelID:  ID of the model.

Exceptions:
-----------

invalid_argument:  if no model with modelID or parameter with paramID
registered. ";

%feature("docstring")  unsetCurrentModel "* Unset the Current Model*
void unsetCurrentModel() ";

%feature("docstring")  isSetCurrentModel "* Checks if a Current Model
is selected* bool isSetCurrentModel() ";

%feature("docstring")  getCurrentModelID "* The ID of the Current
Model* string getCurrentModelID()

Returns the ID of the Current Model; if no Current Model is selected,
the empty string is returned. ";

%feature("docstring")  setParamDouble "* Set the value of a Parameter
of data type double of the Current Model* void setParamDouble(const
string paramID, double val)

Parameters:
-----------

paramID:  ID of the parameter.

val:  The value of the parameter

Exceptions:
-----------

invalid_argument:  if no double parameter with paramID is registered
for the Current Model. ";

%feature("docstring")  setParamInt "* Set the value of a Parameter of
data type int of the Current Model* void setParamInt(const string
paramID, int val)

Parameters:
-----------

paramID:  ID of the parameter.

val:  The value of the parameter

Exceptions:
-----------

invalid_argument:  if no int parameter with paramID is registered for
the Current Model. ";

%feature("docstring")  setParamString "* Set the value of a Parameter
of data type string of the Current Model* void setParamString(const
string paramID, const string val)

Parameters:
-----------

paramID:  ID of the parameter.

val:  The value of the parameter

Exceptions:
-----------

invalid_argument:  if no string parameter with paramID is registered
for the Current Model. ";

%feature("docstring")  setParamEnumIndex "* Set the enumeration index
of a Parameter of data type enum of the Current Model* void
setParamEnumIndex(const string paramID, const unsigned index)

Parameters:
-----------

paramID:  ID of the parameter.

index:  The enumeration index of the parameter

Exceptions:
-----------

invalid_argument:  if no enum parameter with paramID is registered for
the Current Model.

domain_error:  if invalid index is specified. ";

%feature("docstring")  getParamDouble "* Get the value of a Parameter
of data type double of the Current Model* double getParamDouble(const
string paramID)

Returns the value of the parameter.

Parameters:
-----------

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no double parameter with paramID is registered
for the Current Model. ";

%feature("docstring")  getParamInt "* Get the value of a Parameter of
data type int of the Current Model* int getParamInt(const string
paramID)

Returns the value of the parameter.

Parameters:
-----------

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no int parameter with paramID is registered for
the Current Model. ";

%feature("docstring")  getParamString "* Get the value of a Parameter
of data type string of the Current Model* string getParamString(const
string paramID)

Returns the value of the parameter.

Parameters:
-----------

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no string parameter with paramID is registered
for the Current Model. ";

%feature("docstring")  getParamEnumIndex "* Get the enumeration index
of a Parameter of data type enum of the Current Model* unsigned
getParamEnumIndex(const string paramID)

Returns the enumeration index of the parameter.

Parameters:
-----------

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no enum parameter with paramID is registered for
the Current Model. ";

%feature("docstring")  setFunction "* Link a Function to a Result*
void setFunction(const string resultID, const string functionID)

All Function Parameters of the Function are set to their default
values.

Parameters:
-----------

resultID:  ID of the result.

functionID:  ID of the function.

Exceptions:
-----------

invalid_argument:  if no result with resultID is registered.

invalid_argument:  if no function with functionID is registered.

invalid_argument:  if no function and parameter are not of identical
type. ";

%feature("docstring")  unsetFunction "* Unlink any Function that is
linked to a Result* void unsetFunction(const string resultID)

Parameters:
-----------

resultID:  ID of the result.

Exceptions:
-----------

domain_error:  if no function set for specified result. ";

%feature("docstring")  getFunctionID "* Get the ID of the Function
linked to a Result* const string getFunctionID(const string resultID)

If no function is linked to the result, the empty string is returned.

Parameters:
-----------

resultID:  ID of the result.

Exceptions:
-----------

invalid_argument:  if no result with resultID is registered. ";

%feature("docstring")  setFunctionParamDouble "* Set the value of a
Function Parameter of data type double* void
setFunctionParamDouble(const string resultID, const string paramID,
double val)

Parameters:
-----------

resultID:  ID of the result to which the function is linked.

paramID:  ID of the function parameter.

val:  The value of the function parameter

Exceptions:
-----------

invalid_argument:  if no result with resultID is registered.

invalid_argument:  if no double function parameter with paramID is
registered for function linked to the result. ";

%feature("docstring")  setFunctionParamInt "* Set the value of a
Function Parameter of data type int* void setFunctionParamInt(const
string resultID, const string paramID, int val)

Parameters:
-----------

resultID:  ID of the result to which the function is linked.

paramID:  ID of the function parameter.

val:  The value of the function parameter

Exceptions:
-----------

invalid_argument:  if no result with resultID is registered.

invalid_argument:  if no int function parameter with paramID is
registered for function linked to the result. ";

%feature("docstring")  setFunctionParamString "* Set the value of a
Function Parameter of data type string* void
setFunctionParamString(const string resultID, const string paramID,
const string val)

Parameters:
-----------

resultID:  ID of the result to which the function is linked.

paramID:  ID of the function parameter.

val:  The value of the function parameter

Exceptions:
-----------

invalid_argument:  if no result with resultID is registered.

invalid_argument:  if no string function parameter with paramID is
registered for function linked to the result. ";

%feature("docstring")  setFunctionParamEnumIndex "* Set the
enumeration index of a Function Parameter of data type enum* void
setFunctionParamEnumIndex(const string resultID, const string paramID,
unsigned index)

Parameters:
-----------

resultID:  ID of the result to which the function is linked.

paramID:  ID of the function parameter.

index:  The enumeration index of the function parameter

Exceptions:
-----------

invalid_argument:  if no result with resultID is registered.

invalid_argument:  if no enum function parameter with paramID is
registered for function linked to the result.

domain_error:  if invalid index is specified. ";

%feature("docstring")  getFunctionParamDouble "* Get the value of a
Function Parameter of data type double* double
getFunctionParamDouble(const string resultID, const string paramID)

Returns the value of the parameter.

Parameters:
-----------

resultID:  ID of the result to which the function is linked.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no result with resultID is registered.

invalid_argument:  if no double parameter with paramID is registered
for function linked to the result. ";

%feature("docstring")  getFunctionParamInt "* Get the value of a
Function Parameter of data type int* int getFunctionParamInt(const
string resultID, const string paramID)

Returns the value of the parameter.

Parameters:
-----------

resultID:  ID of the result to which the function is linked.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no result with resultID is registered.

invalid_argument:  if no int parameter with paramID is registered for
function linked to the result. ";

%feature("docstring")  getFunctionParamString "* Get the value of a
Function Parameter of data type string* string
getFunctionParamString(const string resultID, const string paramID)

Returns the value of the parameter.

Parameters:
-----------

resultID:  ID of the result to which the function is linked.

paramID:  ID of the parameter.

Exceptions:
-----------

invalid_argument:  if no result with resultID is registered.

invalid_argument:  if no string parameter with paramID is registered
for function linked to the result. ";

%feature("docstring")  getFunctionParamEnumIndex "* Get the
enumeration index of a Function Parameter of data type enum* unsigned
getFunctionParamEnumIndex(const string resultID, const string paramID)

Returns the enumeration index of the Function Parameter.

Parameters:
-----------

resultID:  ID of the result to which the function is linked.

paramID:  ID of the function parameter.

Exceptions:
-----------

invalid_argument:  if no result with resultID is registered.

invalid_argument:  if no enum function parameter with paramID is
registered for function linked to the result. ";

%feature("docstring")  setTdustIdentTemp "* Identifies the Result
tdust with the Result temperature* void setTdustIdentTemp()

Exceptions:
-----------

domain_error:  if \"temperature\" already identified with \"tdust\".
";

%feature("docstring")  unsetTdustIdentTemp "* Unset the
identification of the Result tdust with the Result temperature* void
unsetTdustIdentTemp()

Exceptions:
-----------

domain_error:  if \"tdust\" not identified with \"temperature\". ";

%feature("docstring")  isTdustIdentTemp "* Checks if the Result tdust
is identified with the Result temperature* const bool
isTdustIdentTemp()

Returns True if the Result \"tdust\" is identified with the Result
\"temperature\", else False. ";

%feature("docstring")  setTempIdentTdust "* Identifies the Result
temperature with the Result tdust* void setTempIdentTdust()

Exceptions:
-----------

domain_error:  if \"tdust\" already identified with \"temperature\".
";

%feature("docstring")  unsetTempIdentTdust "* Unset the
identification of the Result temperature with the Result tdust* void
unsetTempIdentTdust()

Exceptions:
-----------

domain_error:  if \"temperature\" not identified with \"tdust\". ";

%feature("docstring")  isTempIdentTdust "* Checks if the Result
temperature is identified with the Result tdust* const bool
isTempIdentTdust()

Returns True if the Result \"temperature\" is identified with the
Result \"tdust\", else False. ";

%feature("docstring")  isCurrentResultModel "* Checks if a Result is
provided by the Current Model* const bool isCurrentResultModel(const
string resultID)

Returns True if the Result is provided by the Current Model, else
False.

Parameters:
-----------

resultID:  ID of the result to which the function is linked.

Exceptions:
-----------

invalid_argument:  if no result with resultID is registered. ";

%feature("docstring")  isCurrentResultFunction "* Checks if a Result
is linked to a Function* const bool isCurrentResultFunction(const
string resultID)

Returns True if the Result is linked to a Function, else False.

Parameters:
-----------

resultID:  ID of the result to which the function is linked.

Exceptions:
-----------

invalid_argument:  if no result with resultID is registered. ";

%feature("docstring")  isConfigurationComplete "* Checks if all
Results are either provided by the Current Model or linked to a
Function* const bool isConfigurationComplete()

Returns True if configuration is complete, else False. ";

%feature("docstring")  finalizeConfiguration "* Finalize the
configuration* void finalizeConfiguration()

Depending on the Current Model and Functions linked to Results,
internal variables of the Current Model are set, data from files may
be read in, etc. ";

%feature("docstring")  isFinalizedConfiguration "* Checks if the
configuration has been finalized* bool isFinalizedConfiguration()

Returns True if configuration has been finalized, else False. ";

%feature("docstring")  temperature "* Gas temperature of current
model at single point* double temperature(double x, double y, double
z)

Returns the gas temperature at x,y,z (K).

Parameters:
-----------

x:  y:  z:  Cartesian coordinates (m). ";

%feature("docstring")  abundance "* Abundance of current model at
single point* double abundance(double x, double y, double z)

Returns the abundance at x,y,z.

Parameters:
-----------

x:  y:  z:  Cartesian coordinates (m). ";

%feature("docstring")  t_dust "* Dust temperature of current model at
single point* double t_dust(double x, double y, double z)

Returns the dust temperature at x,y,z (K).

Parameters:
-----------

x:  y:  z:  Cartesian coordinates (m). ";

%feature("docstring")  doppler "* Doppler b parameter of current
model at single point* double doppler(double x, double y, double z)

Returns the Doppler b-parameter (1/e half-width of line profile) at
x,y,z (m/s).

Parameters:
-----------

x:  y:  z:  Cartesian coordinates (m). ";

%feature("docstring")  velocity "* Velocity of current model at
single point* vector<double> velocity(double x, double y, double z)

Returns the velocity (cartesian components) at x,y,z (m/s).

Parameters:
-----------

x:  y:  z:  Cartesian coordinates (m). ";

%feature("docstring")  bmag "* Magnetic field of current model at
single point* vector<double> bmag(double x, double y, double z)

Returns the magnetic field (cartesian components) at x,y,z (Gauss).

Parameters:
-----------

x:  y:  z:  Cartesian coordinates (m). ";

