////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ARTISTGRID_H
#define ARTISTGRID_H

#include <vector>
#include <string>

using namespace std;

class ArtistGrid {
public:
	virtual ~ArtistGrid() {};

	const virtual int getMaxIndex() const = 0;
	const virtual int getIndex(const vector<double>& coords) const = 0;
};

class ArtistRegularGrid: public ArtistGrid {
public:
	ArtistRegularGrid(const vector<int>& nGridPoints,
			          const int& iCoordSys,
			          const int& iGridLocation);
	virtual ~ArtistRegularGrid() {};

	void addGridPoint(const double& x, const int& iDim);
	void finalizeGrid();

	const virtual int getMaxIndex() const;
	const virtual int getIndex(const vector<double>& coords) const;
private:
	void addPointCenter(const double& x, const int& iDim);
	void addPointInterface(const double& x, const int& iDim);

	int m_iCoordSys;  // 0 Cartesian; 1 Cylindrical; 2 Spherical
	int m_iGridLocation; // 0 Cell centered; 1 Cell interfaces

	vector<vector<double> > m_xGrid; // First index selects axis

	vector<unsigned int> m_iNextIndex; // Next Grid Point to be added here.

	vector<bool> m_hasLowerBound;
	vector<double> m_lowerBound;
	vector<bool> m_hasUpperBound;
	vector<double> m_upperBound;

	// Last indices where getIndex was successfull:
	mutable vector<unsigned int> m_iLastFoundIndices;

	const int binarySearch(const vector<double>& xGrid, const double& x) const;
};

#endif // ARTISTGRID_H

