////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ARTISTTABDATA_H
#define ARTISTTABDATA_H

#include <vector>

using namespace std;

class ArtistTabdata
{
public:
	ArtistTabdata();
	virtual ~ArtistTabdata();
protected:
	unsigned int m_iNextIndex; // Next data will be entered here
};

class ArtistScalarTabdata: public ArtistTabdata {
public:
	ArtistScalarTabdata(const unsigned int& nDataPoints);
	virtual ~ArtistScalarTabdata();

	virtual void addData(const double& data);
	const double& getData(const unsigned int& index) const;
private:
	vector<double> m_data;
};

class ArtistVectorTabdata: public ArtistTabdata
{
public:
	ArtistVectorTabdata(const unsigned int& nDataPoints);
	virtual ~ArtistVectorTabdata();

	virtual void addData(const vector<double>& data);
	const vector<double>& getData(const unsigned int& index) const;
private:
	vector<vector<double> > m_data; // First index grid point; second index vector components
};

class CoordSystem
{
public:
	CoordSystem() {};
	virtual ~CoordSystem() {};

	virtual const vector<double> coordCartToSys(const double& x, const double& y, const double& z) const = 0;
	virtual const vector<double> vecSysToCart(const vector<double>& vecSys, const vector<double>& coordSys) const = 0;
};

class CoordSystemCart: public CoordSystem
{
public:
	virtual const vector<double> coordCartToSys(const double& x, const double& y, const double& z) const;
	virtual const vector<double> vecSysToCart(const vector<double>& vecSys, const vector<double>& coordSys) const;
};

class CoordSystemCyl: public CoordSystem
{
public:
	virtual const vector<double> coordCartToSys(const double& x, const double& y, const double& z) const;
	virtual const vector<double> vecSysToCart(const vector<double>& vecSys, const vector<double>& coordSys) const;
};

class CoordSystemSphe: public CoordSystem
{
public:
	virtual const vector<double> coordCartToSys(const double& x, const double& y, const double& z) const;
	virtual const vector<double> vecSysToCart(const vector<double>& vecSys, const vector<double>& coordSys) const;
};

#endif // ARTISTTABDATA_H

