////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ARTISTTABDATAREADER_H
#define ARTISTTABDATAREADER_H

#include <fstream>
#include <vector>


#include "artistGrid.h"
#include "artistTabdata.h"


class TabdataIndex {
public:
	TabdataIndex(const vector<double>& coords, const unsigned int& index, const bool& isValid);

	const vector<double> coords;
	const unsigned int index;
	const bool isValid;
};



class TabdataReader
{
public:
	TabdataReader(const string& filename);
	virtual ~TabdataReader();


	void openFile();
	void closeFile();

	void parseFile();

	const string getDataname() const;

	const TabdataIndex getIndex(const double& x, const double& y, const double& z) const;
private:
	void parseHeader();
	void parseGrid();
	virtual void parseData() = 0;

protected:
	const string m_filename;
	ifstream m_infile;

	bool m_headerParsed;
	bool m_gridParsed;
	bool m_dataParsed;

	// Header information:
	string m_dataName;         // density, temperature, etc.
	int m_iRegularGrid;        // 0: Regular; 1: Irregular
	int m_iGridType;           // Regular Grids: 0 Cartesian; 1 Cylindrical; 2 Spherical
					           // Irregular Grids: 0 ARM; 1 Delaunay; 2 Voronoi
	int m_iGridLocation;       // 0 Cell centered; 1 Cell interfaces
	vector<int> m_nGridPoints;

	const ArtistGrid* m_pGrid;
	const CoordSystem* m_pCoordSys;
};

class ScalarTabdataReader: public TabdataReader
{
public:
	ScalarTabdataReader(const string& filename);
	virtual ~ScalarTabdataReader();

	const double& getData(const TabdataIndex& index) const;
private:
	virtual void parseData();

	ArtistScalarTabdata* m_pData;
};

class VectorTabdataReader: public TabdataReader
{
public:
	VectorTabdataReader(const string& filename);
	virtual ~VectorTabdataReader();

	const vector<double> getData(const TabdataIndex& index) const;
private:
	virtual void parseData();

	ArtistVectorTabdata* m_pData;
};

#endif // ARTISTTABDATAREADER_H

