////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef FUNCTION_H
#define FUNCTION_H

#include "parameterregistry.h"
#include "iProvider.h"

#include <string>
#include <map>
#include <set>

using namespace std;

struct FunctionReg {
  string name;
  string description;
};

class Function: public IProvider
{
 public:
  Function();
  virtual ~Function() {};

  virtual const FunctionReg getFunctionReg() = 0;
  virtual const set<string> getParamIDs() = 0;
  virtual const ParamReg* getParamReg(string paramID) = 0;

  void initializeParams();

  virtual const string getType() = 0;

  void setParamDouble(string paramID, double val);
  void setParamInt(string paramID, int val);
  void setParamString(string paramID, string val);
  void setParamEnumIndex(string paramID, unsigned index);

  const double getParamDouble(const string paramID);
  const int getParamInt(const string paramID);
  const string getParamString(const string paramID);
  const unsigned getParamEnumIndex(const string paramID);

  void finalizeConfiguration();
  const bool isFinalizedConfiguration();

 protected:
  // Parameter:
  map<string, double> m_paramDouble;
  map<string, int> m_paramInt;
  map<string, string> m_paramString;

  map<string, int> m_paramEnumIndex;
  map<string, double> m_paramEnumValue;

  virtual void onFinalizeConfiguration() = 0;

 private:
  bool m_isFinalized;
};




#endif // FUNCTION_H
