////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef FUNCTIONSCALAR_H
#define FUNCTIONSCALAR_H

#include "function.h"
#include "functionregistry.h"
#include "iProvider.h"

#include <string>
#include <set>

using namespace std;

class FunctionScalar: public Function,
                      public IDensityProvider,
                      public ITemperatureProvider,
                      public IAbundanceProvider,
                      public ITdustProvider,
                      public IDopplerProvider
{
public:
  virtual ~FunctionScalar() {};

  // Function interface:
  virtual const string getType();

  // IProvider interfaces:
  virtual const double
  density(const double x, const double y, const double z);

  virtual const double
  temperature(const double x, const double y, const double z);

  virtual const double
  abundance(const double x, const double y, const double z);

  virtual const double
  t_dust(const double x, const double y, const double z);

  virtual const double
  doppler(const double x, const double y, const double z);

protected:
  virtual void onFinalizeConfiguration() = 0;
  virtual const double value(const double x, const double y, const double z) = 0;
};



class FS_Const: public FunctionScalar
{
public:
  static string functionID;

  virtual ~FS_Const() {};

  virtual const FunctionReg getFunctionReg();
  virtual const set<string> getParamIDs();
  virtual const ParamReg* getParamReg(string paramID);

  virtual const double value(const double x, const double y, const double z);

protected:
  virtual void onFinalizeConfiguration();

private:
  double m_value;
};

class FS_pR: public FunctionScalar
{
public:
  static string functionID;

  virtual ~FS_pR() {};

  virtual const FunctionReg getFunctionReg();
  virtual const set<string> getParamIDs();
  virtual const ParamReg* getParamReg(string paramID);

  virtual const double value(const double x, const double y, const double z);

protected:
  virtual void onFinalizeConfiguration();

private:
  double m_factor;
  double m_exponent;
  double m_offset;
  double m_lowerR;
};

class FS_pRZ: public FunctionScalar
{
public:
  static string functionID;

  virtual ~FS_pRZ() {};

  virtual const FunctionReg getFunctionReg();
  virtual const set<string> getParamIDs();
  virtual const ParamReg* getParamReg(string paramID);

  virtual const double value(const double x, const double y, const double z);

protected:
  virtual void onFinalizeConfiguration();

private:
  double m_factR;
  double m_expR;
  double m_offsetR;
  double m_lowerR;
  double m_factZ;
  double m_expZ;
  double m_offsetZ;
  double m_lowerZ;
};

class FS_pReZ
: public FunctionScalar
{
public:
  static string functionID;

  virtual ~FS_pReZ() {};

  virtual const FunctionReg getFunctionReg();
  virtual const set<string> getParamIDs();
  virtual const ParamReg* getParamReg(string paramID);

  virtual const double value(const double x, const double y, const double z);

protected:
  virtual void onFinalizeConfiguration();

private:
  double m_factR;
  double m_expR;
  double m_offsetR;
  double m_lowerR;
  double m_factZ;
  double m_scaleZ;
  double m_offsetZ;
};

class FS_pRT: public FunctionScalar
{
public:
  static string functionID;

  virtual ~FS_pRT() {};

  virtual const FunctionReg getFunctionReg();
  virtual const set<string> getParamIDs();
  virtual const ParamReg* getParamReg(string paramID);

  virtual const double value(const double x, const double y, const double z);

protected:
  virtual void onFinalizeConfiguration();

private:
  double m_factTheta;
  double m_expTheta;
  double m_offsetTheta;
  double m_lowerTheta;
  double m_factR;
  double m_expR;
  double m_offsetR;
  double m_lowerR;
};





#endif // FUNCTIONSCALAR_H
