////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef FUNCTIONREGISTRY_H
#define FUNCTIONREGISTRY_H

#include <string>
#include <set>
#include <map>
#include <vector>

#include "function.h"
#include "parameterregistry.h"

using namespace std;



class FunctionRegistry
{
  ////////////////////////////////////////////////////////////////////
  //! Implementation of the Singleton pattern.
  /**
   * @name Singleton
   */
  //@{
public:
  static FunctionRegistry* getInstance();
  virtual ~FunctionRegistry();
protected:
  FunctionRegistry();
private:
  static FunctionRegistry* m_theFunctionRegistry;
  //@}

  // Other:
public:
  void registerFunctions();

  bool isRegisteredFunction(const string& functionID) const;
  set<string> getFunctionIDs() const;
  string getFunctionName(const string& functionID) const;
  string getFunctionDesc(const string& functionID) const;
  string getFunctionType(const string& functionID) const;

  bool isRegisteredParam(const string& functionID, const string& paramID) const;
  set<string> getParamIDs(const string& functionID) const;
  string getParamType(const string& functionID, const string& paramID) const;
  string getParamName(const string& functionID, const string& paramID) const;
  string getParamDesc(const string& functionID, const string& paramID) const;
  int getParamDefValInt(const string& functionID, const string& paramID) const;
  double getParamDefValDouble(const string& functionID, const string& paramID) const;
  string getParamDefValString(const string& functionID, const string& paramID) const;
  vector<double> getParamEnumValues(const string& functionID, const string& paramID) const;
  unsigned getParamEnumDefIndex(const string& functionID, const string& paramID) const;

  Function* createFunction(const string functionID) const;

private:
  // Function registry:
  set<string> m_functionIDs;
  map<string, FunctionReg> m_functionRegs;

  void registerFunction(const string& functionID);
  void addFunctionReg(const string& functionID, const FunctionReg functionReg);
  const FunctionReg& getFunctionReg(const string& functionID) const;

  // Parameter registries:
  map<string, ParameterRegistry> m_parameterRegistries; // Key: functionID
  const ParameterRegistry& getParameterRegistry(const string& functionID) const;
};




#endif // FUNCTIONREGISTRY_H
