////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#include "iProvider.h"

using namespace std;

const string IDensityProvider::providerID = "density";
const string IDensityProvider::providerName = "Density";
const string IDensityProvider::providerDesc = "H2 density";
const string IDensityProvider::providerUnit = "m^-3";
const string IDensityProvider::providerType =  providerTypeScalar;

bool
isDensityProvider(IProvider* ip)
{
	IDensityProvider *idcp = dynamic_cast<IDensityProvider*>(ip);
	if (idcp)
		return true;
	return false;
}

const string ITemperatureProvider::providerID = "temperature";
const string ITemperatureProvider::providerName = "Temperature";
const string ITemperatureProvider::providerDesc = "Gas temperature";
const string ITemperatureProvider::providerUnit = "K";
const string ITemperatureProvider::providerType =  providerTypeScalar;

bool
isTemperatureProvider(IProvider* ip)
{
	ITemperatureProvider *idcp = dynamic_cast<ITemperatureProvider*>(ip);
	if (idcp)
		return true;
	return false;
}

const string IAbundanceProvider::providerID = "abundance";
const string IAbundanceProvider::providerName = "Abundance";
const string IAbundanceProvider::providerDesc = "Molecular abundance";
const string IAbundanceProvider::providerUnit = "";
const string IAbundanceProvider::providerType =  providerTypeScalar;

bool
isAbundanceProvider(IProvider* ip)
{
	IAbundanceProvider *idcp = dynamic_cast<IAbundanceProvider*>(ip);
	if (idcp)
		return true;
	return false;
}

const string ITdustProvider::providerID = "tdust";
const string ITdustProvider::providerName = "Tdust";
const string ITdustProvider::providerDesc = "Dust temperature";
const string ITdustProvider::providerUnit = "K";
const string ITdustProvider::providerType =  providerTypeScalar;

bool
isTdustProvider(IProvider* ip)
{
	ITdustProvider *idcp = dynamic_cast<ITdustProvider*>(ip);
	if (idcp)
		return true;
	return false;
}

const string IDopplerProvider::providerID = "doppler";
const string IDopplerProvider::providerName = "Doppler";
const string IDopplerProvider::providerDesc = "Doppler b-parameter";
const string IDopplerProvider::providerUnit = "m/s";
const string IDopplerProvider::providerType =  providerTypeScalar;

bool
isDopplerProvider(IProvider* ip)
{
	IDopplerProvider *idcp = dynamic_cast<IDopplerProvider*>(ip);
	if (idcp)
		return true;
	return false;
}

const string IVelocityProvider::providerID = "velocity";
const string IVelocityProvider::providerName = "Velocity";
const string IVelocityProvider::providerDesc = "Velocity";
const string IVelocityProvider::providerUnit = "m/s";
const string IVelocityProvider::providerType =  providerTypeVector;

bool
isVelocityProvider(IProvider* ip)
{
	IVelocityProvider *idcp = dynamic_cast<IVelocityProvider*>(ip);
	if (idcp)
		return true;
	return false;
}

const string IBmagProvider::providerID = "bmag";
const string IBmagProvider::providerName = "B magnetic";
const string IBmagProvider::providerDesc = "Magnetic field strength";
const string IBmagProvider::providerUnit = "T";
const string IBmagProvider::providerType =  providerTypeVector;

bool
isBmagProvider(IProvider* ip)
{
	IBmagProvider *idcp = dynamic_cast<IBmagProvider*>(ip);
	if (idcp)
		return true;
	return false;
}


TempTdustAdaptor::TempTdustAdaptor(ITemperatureProvider* pTemperatureProvider) :
	m_pTemperatureProvider(pTemperatureProvider)
{}

TempTdustAdaptor::~TempTdustAdaptor()
{
	m_pTemperatureProvider = NULL;
}

const double
TempTdustAdaptor::t_dust(const double x, const double y, const double z)
{
	return m_pTemperatureProvider->temperature(x, y, z);
}


TdustTempAdaptor::TdustTempAdaptor(ITdustProvider* pTdustProvider) :
	m_pTdustProvider(pTdustProvider)
{}

TdustTempAdaptor::~TdustTempAdaptor()
{
	m_pTdustProvider = NULL;
}

const double
TdustTempAdaptor::temperature(const double x, const double y, const double z)
{
	return m_pTdustProvider->t_dust(x, y, z);
}


