////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#include <string>
#include <set>

#include "icModellib.h"
#include "modellib.h"



#include <iostream>

// Exploration:
// Models:
int
getNumModels()
{
  return ModelLib::getInstance()->getModelIDs().size();
}

char**
getModelIDs()
{
  char** cModelIDs; 
  cModelIDs = new char*[getNumModels()];

  set<string> modelIDs = ModelLib::getInstance()->getModelIDs();
  set<string>::iterator itModel;

  int iPos = 0;
  for (itModel = modelIDs.begin(); itModel != modelIDs.end(); ++itModel) {
    string modelID = *itModel;
    cModelIDs[iPos++] = (char*)(modelID.c_str());
  }
  return cModelIDs;

}

int
isRegisteredModel(const char* modelID)
{
  if (ModelLib::getInstance()->isRegisteredModel(string(modelID))) {
    return 1;
  } else {
    return 0;
  }
}

char*
getModelName(const char* modelID)
{
  return (char*)(ModelLib::getInstance()->getModelName(modelID).c_str());
}

char*
getModelDesc(const char* modelID)
{
  return (char*)(ModelLib::getInstance()->getModelDesc(modelID).c_str());
}

char*
getModelBibref(const char* modelID)
{
  return (char*)(ModelLib::getInstance()->getModelBibref(modelID).c_str());
}

int
getNumParams(const char* modelID)
{
  return ModelLib::getInstance()->getParamIDs(modelID).size();
}


char **
getParamIDs(const char* modelID)
{
  char** cParamIDs;

  cParamIDs = new char*[getNumParams(modelID)]; 

  set<string> paramIDs = ModelLib::getInstance()->getParamIDs(modelID);
  set<string>::iterator itParam;
  
  int iPos = 0;
  for (itParam = paramIDs.begin(); itParam != paramIDs.end(); ++itParam) {
    string paramID = *itParam;
    cParamIDs[iPos++] = (char*)(paramID.c_str());
  }
  return cParamIDs;
}

int
isRegisteredParam(const char* modelID, const char* paramID)
{
  if (ModelLib::getInstance()->isRegisteredParam(string(modelID), string(paramID))) {
    return 1;
  } else {
    return 0;
  }
}

char*
getParamType(const char* modelID, const char* paramID)
{
  return (char*)(ModelLib::getInstance()->getParamType(modelID, paramID).c_str());
}

char*
getParamName(const char* modelID, const char* paramID)
{
  return (char*)(ModelLib::getInstance()->getParamName(modelID, paramID).c_str());
}

char*
getParamDesc(const char* modelID, const char* paramID)
{
  return (char*)(ModelLib::getInstance()->getParamDesc(modelID, paramID).c_str());
}

char*
getParamUnit(const char* modelID, const char* paramID)
{
  return (char*)(ModelLib::getInstance()->getParamUnit(modelID, paramID).c_str());
}

int
getParamDefValInt(const char* modelID, const char* paramID)
{
  return ModelLib::getInstance()->getParamDefValInt(modelID, paramID);
}

double
getParamDefValDouble(const char* modelID, const char* paramID)
{
  return ModelLib::getInstance()->getParamDefValDouble(modelID, paramID);
}

char*
getParamDefValString(const char* modelID, const char* paramID)
{
  return (char*)(ModelLib::getInstance()->getParamDefValString(modelID, paramID).c_str());
}

int
getNumParamEnumValues(const char* modelID, const char* paramID)
{
  return ModelLib::getInstance()->getParamEnumValues(modelID, paramID).size();
}

double*
getParamEnumValues(const char* modelID, const char* paramID)
{
  vector<double> vValues = ModelLib::getInstance()->getParamEnumValues(modelID, paramID);
  double* values = new double[vValues.size()];
  for (unsigned int i=0; i<vValues.size(); i++)
    values[i] = vValues[i];
  return values;
}

unsigned
getParamEnumDefIndex(const char* modelID, const char* paramID)
{
  return ModelLib::getInstance()->getParamEnumDefIndex(modelID, paramID);
}






// Results:
int
getNumResults()
{
  return ModelLib::getInstance()->getProviderIDs().size();
}

char**
getResultIDs()
{
  char** cResultIDs;
  cResultIDs = new char*[getNumResults()];

  set<string> resultIDs = ModelLib::getInstance()->getProviderIDs();
  set<string>::iterator itResult;

  int iPos = 0;
  for (itResult = resultIDs.begin(); itResult != resultIDs.end(); ++itResult) {
    string resultID = *itResult;
    cResultIDs[iPos++] = (char*)(resultID.c_str());
  }
  return cResultIDs;

}

int
isRegisteredResult(const char* resultID)
{
  if (ModelLib::getInstance()->isRegisteredProvider(string(resultID))) {
    return 1;
  } else {
    return 0;
  }
}

char*
getResultName(const char* resultID)
{
  return (char*)(ModelLib::getInstance()->getProviderName(resultID).c_str());
}

char*
getResultDesc(const char* resultID)
{
  return (char*)(ModelLib::getInstance()->getProviderDesc(resultID).c_str());
}

char*
getResultUnit(const char* resultID)
{
  return (char*)(ModelLib::getInstance()->getProviderUnit(resultID).c_str());
}

char*
getResultType(const char* resultID)
{
  return (char*)(ModelLib::getInstance()->getProviderType(resultID).c_str());
}

int
getNumModelResults(const char* modelID)
{
  return ModelLib::getInstance()->getModelProviderIDs(modelID).size();
}

char **
getModelResultIDs(const char* modelID)
{
  char** cResultIDs;

  cResultIDs = new char*[getNumModelResults(modelID)];

  set<string> resultIDs = ModelLib::getInstance()->getModelProviderIDs(modelID);
  set<string>::iterator itResult;

  int iPos = 0;
  for (itResult = resultIDs.begin(); itResult != resultIDs.end(); ++itResult) {
    string resultID = *itResult;
    cResultIDs[iPos++] = (char*)(resultID.c_str());
  }
  return cResultIDs;
}


int
isResultModel(const char* resultID, const char* modelID)
{
  if (ModelLib::getInstance()->isProviderModel(string(resultID), string(modelID))) {
    return 1;
  } else {
    return 0;
  }
}

// Functions:
int
getNumFunctions()
{
  return ModelLib::getInstance()->getFunctionIDs().size();
}

char**
getFunctionIDs()
{
  char** cFunctionIDs;
  cFunctionIDs = new char*[getNumFunctions()];

  set<string> functionIDs = ModelLib::getInstance()->getFunctionIDs();
  set<string>::iterator itFunction;

  int iPos = 0;
  for (itFunction = functionIDs.begin(); itFunction != functionIDs.end(); ++itFunction) {
    string functionID = *itFunction;
    cFunctionIDs[iPos++] = (char*)(functionID.c_str());
  }
  return cFunctionIDs;

}

int
isRegisteredFunction(const char* functionID)
{
  if (ModelLib::getInstance()->isRegisteredFunction(string(functionID))) {
    return 1;
  } else {
    return 0;
  }
}

char*
getFunctionName(const char* functionID)
{
  return (char*)(ModelLib::getInstance()->getFunctionName(functionID).c_str());
}

char*
getFunctionDesc(const char* functionID)
{
  return (char*)(ModelLib::getInstance()->getFunctionDesc(functionID).c_str());
}

char*
getFunctionType(const char* functionID)
{
  return (char*)(ModelLib::getInstance()->getFunctionType(functionID).c_str());
}

int
getNumFunctionResults(const char* functionID)
{
  return ModelLib::getInstance()->getFunctionProviderIDs(functionID).size();
}

char **
getFunctionResultIDs(const char* functionID)
{
  char** cResultIDs;

  cResultIDs = new char*[getNumFunctionResults(functionID)];

  set<string> resultIDs = ModelLib::getInstance()->getFunctionProviderIDs(functionID);
  set<string>::iterator itResult;

  int iPos = 0;
  for (itResult = resultIDs.begin(); itResult != resultIDs.end(); ++itResult) {
    string resultID = *itResult;
    cResultIDs[iPos++] = (char*)(resultID.c_str());
  }
  return cResultIDs;
}

int
getNumResultFunctions(const char* resultID)
{
  return ModelLib::getInstance()->getProviderFunctionIDs(resultID).size();
}

char **
getResultFunctionIDs(const char* resultID)
{
  char** cFunctionIDs;

  cFunctionIDs = new char*[getNumResultFunctions(resultID)];

  set<string> functionIDs = ModelLib::getInstance()->getProviderFunctionIDs(resultID);
  set<string>::iterator itFunction;

  int iPos = 0;
  for (itFunction = functionIDs.begin(); itFunction != functionIDs.end(); ++itFunction) {
    string functionID = *itFunction;
    cFunctionIDs[iPos++] = (char*)(functionID.c_str());
  }
  return cFunctionIDs;
}

int
isResultFunction(const char* resultID, const char* functionID)
{
  if (ModelLib::getInstance()->isProviderFunction(string(resultID), string(functionID))) {
    return 1;
  } else {
    return 0;
  }
}



int
getNumFunctionParams(const char* functionID)
{
  return ModelLib::getInstance()->getFunctionParamIDs(functionID).size();
}

char **
getFunctionParamIDs(const char* functionID)
{
  char** cParamIDs;

  cParamIDs = new char*[getNumFunctionParams(functionID)];

  set<string> paramIDs = ModelLib::getInstance()->getFunctionParamIDs(functionID);
  set<string>::iterator itParam;

  int iPos = 0;
  for (itParam = paramIDs.begin(); itParam != paramIDs.end(); ++itParam) {
    string paramID = *itParam;
    cParamIDs[iPos++] = (char*)(paramID.c_str());
  }
  return cParamIDs;
}

int
isRegisteredFunctionParam(const char* functionID, const char* paramID)
{
  if (ModelLib::getInstance()->isRegisteredFunctionParam(string(functionID), string(paramID))) {
    return 1;
  } else {
    return 0;
  }
}

char*
getFunctionParamType(const char* functionID, const char* paramID)
{
  return (char*)(ModelLib::getInstance()->getFunctionParamType(functionID, paramID).c_str());
}

char*
getFunctionParamName(const char* functionID, const char* paramID)
{
  return (char*)(ModelLib::getInstance()->getFunctionParamName(functionID, paramID).c_str());
}

char*
getFunctionParamDesc(const char* functionID, const char* paramID)
{
  return (char*)(ModelLib::getInstance()->getFunctionParamDesc(functionID, paramID).c_str());
}

int
getFunctionParamDefValInt(const char* functionID, const char* paramID)
{
  return ModelLib::getInstance()->getFunctionParamDefValInt(functionID, paramID);
}

double
getFunctionParamDefValDouble(const char* functionID, const char* paramID)
{
  return ModelLib::getInstance()->getFunctionParamDefValDouble(functionID, paramID);
}

char*
getFunctionParamDefValString(const char* functionID, const char* paramID)
{
  return (char*)(ModelLib::getInstance()->getFunctionParamDefValString(functionID, paramID).c_str());
}

int
getNumFunctionParamEnumValues(const char* functionID, const char* paramID)
{
  return ModelLib::getInstance()->getFunctionParamEnumValues(functionID, paramID).size();
}

double*
getFunctionParamEnumValues(const char* functionID, const char* paramID)
{
  vector<double> vValues = ModelLib::getInstance()->getFunctionParamEnumValues(functionID, paramID);
  double* values = new double[vValues.size()];
  for (unsigned int i=0; i<vValues.size(); i++)
    values[i] = vValues[i];
  return values;
}

unsigned
getFunctionParamEnumDefIndex(const char* functionID, const char* paramID)
{
  return ModelLib::getInstance()->getFunctionParamEnumDefIndex(functionID, paramID);
}




// Configuration:

void setCurrentModel(const char* modelID)
{
  ModelLib::getInstance()->setCurrentModel(string(modelID));
}

void unsetCurrentModel()
{
  ModelLib::getInstance()->unsetCurrentModel();
}

int isSetCurrentModel()
{
  if (ModelLib::getInstance()->isSetCurrentModel()) {
    return 1;
  } else {
    return 0;
  }
}

char*
getCurrentModelID()
{
  return (char*)(ModelLib::getInstance()->getCurrentModelID().c_str());
}



void setParamDouble(const char* paramID, double val)
{
  ModelLib::getInstance()->setParamDouble(paramID, val);
}

void setParamInt(const char* paramID, int val)
{
  ModelLib::getInstance()->setParamInt(paramID, val);
}

void setParamString(const char* paramID, const char* val)
{
  ModelLib::getInstance()->setParamString(paramID, val);
}

void setParamEnumIndex(const char* paramID, const unsigned index)
{
  ModelLib::getInstance()->setParamEnumIndex(paramID, index);
}

double getParamDouble(const char* paramID)
{
  return ModelLib::getInstance()->getParamDouble(paramID);
}

int getParamInt(const char* paramID)
{
  return ModelLib::getInstance()->getParamInt(paramID);
}

char* getParamString(const char* paramID)
{
  return (char*)(ModelLib::getInstance()->getParamString(paramID).c_str());
}

unsigned getParamEnumIndex(const char* paramID)
{
  return ModelLib::getInstance()->getParamEnumIndex(paramID);
}

int isConfigurationComplete()
{
	if (ModelLib::getInstance()->isConfigurationComplete())
		return 1;
	else
		return 0;
}

void finalizeConfiguration()
{
  ModelLib::getInstance()->finalizeConfiguration();
}

int isFinalizedConfiguration()
{
  if (ModelLib::getInstance()->isFinalizedConfiguration())
    return 1;
  else
    return 0;
}

void setFunction(const char* resultID, const char* functionID)
{
  ModelLib::getInstance()->setFunction(resultID, functionID);
}

void unsetFunction(const char* resultID)
{
  ModelLib::getInstance()->unsetFunction(resultID);
}

char* getFunctionID(const char* resultID)
{
  return (char*)(ModelLib::getInstance()->getFunctionID(resultID).c_str());
}

void setFunctionParamDouble(const char* resultID, const char* paramID, double val)
{
  ModelLib::getInstance()->setFunctionParamDouble(resultID, paramID, val);
}

void setFunctionParamInt(const char* resultID, const char* paramID, int val)
{
  ModelLib::getInstance()->setFunctionParamInt(resultID, paramID, val);
}

void setFunctionParamString(const char* resultID, const char* paramID, const char* val)
{
  ModelLib::getInstance()->setFunctionParamString(resultID, paramID, val);
}

void setFunctionParamEnumIndex(const char* resultID, const char* paramID, const unsigned index)
{
  ModelLib::getInstance()->setFunctionParamEnumIndex(resultID, paramID, index);
}

double getFunctionParamDouble(const char* resultID, const char* paramID)
{
  return ModelLib::getInstance()->getFunctionParamDouble(resultID, paramID);
}

int getFunctionParamInt(const char* resultID, const char* paramID)
{
  return ModelLib::getInstance()->getFunctionParamInt(resultID, paramID);
}

char* getFunctionParamString(const char* resultID, const char* paramID)
{
  return (char*)(ModelLib::getInstance()->getFunctionParamString(resultID, paramID).c_str());
}

unsigned getFunctionParamEnumIndex(const char* resultID, const char* paramID)
{
  return ModelLib::getInstance()->getFunctionParamEnumIndex(resultID, paramID);
}

/* Identification of Tdust with Temperature: */
void setTdustIdentTemp()
{
	ModelLib::getInstance()->setTdustIdentTemp();
}

void unsetTdustIdentTemp()
{
	ModelLib::getInstance()->unsetTdustIdentTemp();
}

int isTdustIdentTemp()
{
	if (ModelLib::getInstance()->isTdustIdentTemp())
		return 1;
	return 0;
}

/* Identification of Temperature with Tdust: */
void setTempIdentTdust()
{
	ModelLib::getInstance()->setTempIdentTdust();
}

void unsetTempIdentTdust()
{
	ModelLib::getInstance()->unsetTempIdentTdust();
}

int isTempIdentTdust()
{
	if (ModelLib::getInstance()->isTempIdentTdust())
		return 1;
	return 0;
}


/* Results: */
int isCurrentResultModel(const char* resultID)
{
  if (ModelLib::getInstance()->isCurrentProviderModel(resultID))
    return 1;
  return 0;
}

int isCurrentResultFunction(const char* resultID)
{
  if (ModelLib::getInstance()->isCurrentProviderFunction(resultID))
    return 1;
  return 0;
}



// Usage:

double modellibDensity(double x, double y, double z)
{
  return ModelLib::getInstance()->density(x, y, z);
}

double modellibTemperature(double x, double y, double z)
{
  return ModelLib::getInstance()->temperature(x, y, z);
}


double modellibAbundance(double x, double y, double z)
{
  return ModelLib::getInstance()->abundance(x, y, z);
}


double modellibTdust(double x, double y, double z){
  return ModelLib::getInstance()->t_dust(x, y, z);
}


double modellibDoppler(double x, double y, double z)
{
  return ModelLib::getInstance()->doppler(x, y, z);
}


void modellibVelocity(double x, double y, double z, double* v)
{
  ModelLib::getInstance()->velocity(x, y, z, v);
}

void modellibBmag(double x, double y, double z, double* b)
{
  ModelLib::getInstance()->bmag(x, y, z, b);
}
