////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef MODEL_ALLEN_H
#define MODEL_ALLEN_H

#include "model.h"

#include <set>

class ModelAllen : public Model,
public IDensityProvider,
public ITemperatureProvider,
public IVelocityProvider,
public IBmagProvider
{
  // Model interface:
public:
  ModelAllen();
  virtual ~ModelAllen();

  static string modelID;
  const ModelReg getModelReg();
  const set<string> getParamIDs();
  const ParamReg* getParamReg(string paramID);

protected:
  virtual void onFinalizeConfiguration();

  // IProvider interfaces:
public:
  virtual const double
  density(const double x, const double y, const double z);

  virtual const double
  temperature(const double x, const double y, const double z);

  virtual const void
  velocity(const double x, const double y, const double z, double* v);

  virtual const void
  bmag(const double x, const double y, const double z, double* b);

private:
  double m_v0;
  double m_t0;
  double m_rn;
  double m_T;
  double m_x;
  double m_v;
  double m_r0;
  double m_n0;
  double m_b0;
};

#endif //MODEL_ALLEN_H
