////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Attila Juhasz
// Sterrewacht Leiden
// Universiteit Leiden
// juhasz@strw.leidenuniv.nl
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef MODEL_CG97_H
#define MODEL_CG97_H

#include "model.h"

#include <set>

class ModelCG97 : public Model,
public IDensityProvider,
public ITdustProvider,
public IVelocityProvider
{
  // Model interface:
public:
  ModelCG97();
  virtual ~ModelCG97();

  static string modelID;
  const ModelReg getModelReg();
  const set<string> getParamIDs();
  const ParamReg* getParamReg(string paramID);

protected:
  virtual void onFinalizeConfiguration();

  // IProvider interfaces:
public:
  virtual const double
  density(const double x, const double y, const double z);

  virtual const double
  t_dust(const double x, const double y, const double z);

  virtual const void
  velocity(const double x, const double y, const double z, double* v);


private:

#define npgrid 300
  double my_r[npgrid];
  double my_ri[npgrid+2];
  double my_hp[npgrid];
  double my_alpha[npgrid];
  double my_ti[npgrid];
  double my_ts[npgrid];

  double m_rin;
  double m_rout;
  double m_plsig1;
  double m_hph;
  double m_sig0;
  double m_tstar;
  double m_rstar;
  double m_mstar;
  double m_bgdens;
};

#endif //MODEL_CG97_H
