////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Attila Juhasz
// Sterrewacht Leiden
// Universiteit Leiden
// juhasz@strw.leidenuniv.nl
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef MODEL_DDN01_H
#define MODEL_DDN01_H

#include "model.h"

#include <set>

class ModelDDN01 : public Model,
public IDensityProvider,
public ITdustProvider,
public IVelocityProvider
{
  // Model interface:
public:
  ModelDDN01();
  virtual ~ModelDDN01();

  static string modelID;
  const ModelReg getModelReg();
  const set<string> getParamIDs();
  const ParamReg* getParamReg(string paramID);

protected:
  virtual void onFinalizeConfiguration();

  // IProvider interfaces:
public:
  virtual const double
  density(const double x, const double y, const double z);

  virtual const double
  t_dust(const double x, const double y, const double z);

  virtual const void
  velocity(const double x, const double y, const double z, double* v);


private:

#define nwav 100
#define nrmod 100
  double my_r[nrmod];
  double my_ri[nrmod+1];
  double my_hp[nrmod];
  double my_hs[nrmod];
  double my_ti[nrmod];
  double my_ts[nrmod];
  double my_sig0;

  double m_rin;
  double m_rout;
  double m_mdisk;
  double m_plsig1;
  double m_mstar;
  double m_rstar;
  double m_tstar;
  double m_bgdens;
  const char *m_dustopac_fname;
};

#endif //MODEL_DDN01_H
