////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Wouter Vlemmings
// Chalmers University of Technology
// Onsala Space Observatory
// wouter.vlemmings.at.chalmers.se	
//
// and
// Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef MODEL_MAMON88_H
#define MODEL_MAMON88_H

#include "model.h"

#include <set>

class ModelMamon88 : public Model,
public IDensityProvider,
public ITemperatureProvider,
public IAbundanceProvider,
public IVelocityProvider
{
  // Model interface:
public:
  // ADJUST: Rename constructor and destructor
  ModelMamon88();
  virtual ~ModelMamon88();

  static string modelID;
  const ModelReg getModelReg();
  const set<string> getParamIDs();
  const ParamReg* getParamReg(string paramID);

protected:
  virtual void onFinalizeConfiguration();

  // IProvider interfaces:
public:
  virtual const double
  density(const double x, const double y, const double z);

  // Related with ITemperatureProvider
  virtual const double
  temperature(const double x, const double y, const double z);

  // Related with IAbundanceProvider
  virtual const double
  abundance(const double x, const double y, const double z);

  // Related with IVelocityProvider
  virtual const void
  velocity(const double x, const double y, const double z, double* v);

private:
  double m_mdot;
  double m_rin;
  double m_ve;
  double m_tin;
  double m_ab0;
  double m_alpha;
  double m_rhalf;
};

#endif //MODEL_MAMON88_H
