////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef MODEL_TABDATA_H
#define MODEL_TABDATA_H

#include "model.h"

#include <set>

#include "artistTabdataReader.h"

class ModelTabdata : public Model,
public IDensityProvider,
public ITemperatureProvider,
public IAbundanceProvider,
public ITdustProvider,
public IDopplerProvider,
public IVelocityProvider,
public IBmagProvider
{
  // Model interface:
public:
  ModelTabdata();
  virtual ~ModelTabdata();

  static string modelID;
  const ModelReg getModelReg();
  const set<string> getParamIDs();
  const ParamReg* getParamReg(string paramID);

protected:
  virtual void onFinalizeConfiguration();

  // IProvider interfaces:
public:
  virtual const double
  density(const double x, const double y, const double z);

  virtual const double
  temperature(const double x, const double y, const double z);

  virtual const double
  abundance(const double x, const double y, const double z);

  virtual const double
  t_dust(const double x, const double y, const double z);

  virtual const double
  doppler(const double x, const double y, const double z);

  virtual const void
  velocity(const double x, const double y, const double z, double* v);

  virtual const void
  bmag(const double x, const double y, const double z, double* bmag);
private:
  ScalarTabdataReader* m_pDensityReader;
  ScalarTabdataReader* m_pTemperatureReader;
  ScalarTabdataReader* m_pAbundanceReader;
  ScalarTabdataReader* m_pTdustReader;
  ScalarTabdataReader* m_pDopplerReader;
  VectorTabdataReader* m_pVelocityReader;
  VectorTabdataReader* m_pBmagReader;
};

#endif //MODEL_TABDATA_H
