////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef MODEL_USER_H
#define MODEL_USER_H

#include "model.h"

#include <set>

class ModelUser : public Model,
public IDensityProvider,
public ITemperatureProvider,
public IAbundanceProvider,
public ITdustProvider,
public IDopplerProvider,
public IVelocityProvider,
public IBmagProvider
{
  // Model interface:
public:
  ModelUser();
  virtual ~ModelUser();

  static string modelID;
  const ModelReg getModelReg();
  const set<string> getParamIDs();
  const ParamReg* getParamReg(string paramID);

protected:
  virtual void onFinalizeConfiguration();

  // IProvider interfaces:
public:
  virtual const double
  density(const double x, const double y, const double z);

  virtual const double
  temperature(const double x, const double y, const double z);

  virtual const double
  abundance(const double x, const double y, const double z);

  virtual const double
  t_dust(const double x, const double y, const double z);

  virtual const double
  doppler(const double x, const double y, const double z);

  virtual const void
  velocity(const double x, const double y, const double z, double* v);

  virtual const void
  bmag(const double x, const double y, const double z, double* b);

private:
  double m_paramDbl1;
  double m_paramDbl2;
  double m_paramDbl3;
  double m_paramDbl4;
  double m_paramDbl5;

  int m_paramInt1;
  int m_paramInt2;
  int m_paramInt3;
  int m_paramInt4;
  int m_paramInt5;

  string m_paramStr1;
  string m_paramStr2;
  string m_paramStr3;
  string m_paramStr4;
  string m_paramStr5;
};

#endif //MODEL_USER_H
