/*
////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////
*/

%define DOCSTRING
"ARTIST Model Library\n(C) 2012 R.Schaaf, Universitaet Bonn\nLicense GPLv3"
%enddef
%module(docstring=DOCSTRING) modellib

%pythoncode %{
__version__ = "1.0.0-rc1"
%}

%include "icppModellib.i"

%{
#include "icppModellib.h"
%}

%include "std_string.i"
%include "std_set.i"
%include "std_vector.i"
%include "exception.i"

%exception {
  try {
    $action
  } catch (const std::exception& e) {
    SWIG_exception(SWIG_RuntimeError, e.what());
  }
}

%template(StringSet) std::set<std::string>;
%template(DoubleVector) std::vector<double>;

/* Exploration: */
/* Models: */
bool isRegisteredModel(const std::string modelID);
std::set<std::string> getModelIDs();
std::string getModelName(const std::string modelID);
std::string getModelDesc(const std::string modelID);
std::string getModelBibref(const std::string modelID);

bool isRegisteredParam(const std::string modelID, std::string paramID);
std::set<std::string> getParamIDs(const std::string modelID);
std::string getParamType(const std::string modelID, const std::string paramID);
std::string getParamName(const std::string modelID, const std::string paramID);
std::string getParamDesc(const std::string modelID, const std::string paramID);
std::string getParamUnit(const std::string modelID, const std::string paramID);
int getParamDefValInt(const std::string modelID, const std::string paramID);
double getParamDefValDouble(const std::string modelID, const std::string paramID);
std::string getParamDefValString(const std::string modelID, const std::string paramID);
std::vector<double> getParamEnumValues(const std::string modelID, const std::string paramID);
unsigned getParamEnumDefIndex(const std::string modelID, const std::string paramID);

/* Results: */
bool isRegisteredResult(const std::string resultID);
std::set<std::string> getResultIDs();
std::string getResultName(const std::string resultID);
std::string getResultDesc(const std::string resultID);
std::string getResultUnit(const std::string resultID);
std::string getResultType(const std::string resultID);

std::set<std::string> getModelResultIDs(const std::string modelID);
bool isResultModel(const std::string resultID, const std::string modelID);

/* Functions: */
bool isRegisteredFunction(const std::string functionID);
std::set<std::string> getFunctionIDs();
std::string getFunctionName(const std::string functionID);
std::string getFunctionDesc(const std::string functionID);
std::string getFunctionType(const std::string functionID);

std::set<std::string> getFunctionResultIDs(const std::string functionID);
std::set<std::string> getResultFunctionIDs(const std::string resultID);
bool isResultFunction(const std::string resultID, const std::string functionID);

bool isRegisteredFunctionParam(const std::string functionID, std::string paramID);
std::set<std::string> getFunctionParamIDs(const std::string functionID);
std::string getFunctionParamType(const std::string functionID, const std::string paramID);
std::string getFunctionParamName(const std::string functionID, const std::string paramID);
std::string getFunctionParamDesc(const std::string functionID, const std::string paramID);
int getFunctionParamDefValInt(const std::string functionID, const std::string paramID);
double getFunctionParamDefValDouble(const std::string functionID, const std::string paramID);
std::string getFunctionParamDefValString(const std::string functionID, const std::string paramID);
std::vector<double> getFunctionParamEnumValues(const std::string functionID, const std::string paramID);
unsigned getFunctionParamEnumDefIndex(const std::string functionID, const std::string paramID);

/* Configuration: */
/* Model: */
void setCurrentModel(const std::string modelID);
void unsetCurrentModel();
bool isSetCurrentModel();
std::string getCurrentModelID();

void setParamDouble(const std::string paramID, double val);
void setParamInt(const std::string paramID, int val);
void setParamString(const std::string paramID, const std::string val);
void setParamEnumIndex(const std::string paramID, unsigned index);

double getParamDouble(const std::string paramID);
int getParamInt(const std::string paramID);
std::string getParamString(const std::string paramID);
unsigned getParamEnumIndex(const std::string paramID);

/* Functions: */
void setFunction(const std::string resultID, const std::string functionID);
void unsetFunction(const std::string resultID);
std::string getFunctionID(const std::string resultID);

void setFunctionParamDouble(const std::string resultID, const std::string paramID, double val);
void setFunctionParamInt(const std::string resultID, const std::string paramID, int val);
void setFunctionParamString(const std::string resultID, const std::string paramID, const std::string val);
void setFunctionParamEnumIndex(const std::string resultID, const std::string paramID, unsigned index);

double getFunctionParamDouble(const std::string resultID, const std::string paramID);
int getFunctionParamInt(const std::string resultID, const std::string paramID);
std::string getFunctionParamString(const std::string resultID, const std::string paramID);
unsigned getFunctionParamEnumIndex(const std::string resultID, const std::string paramID);

/* Identification of Tdust with Temperature: */
void setTdustIdentTemp();
void unsetTdustIdentTemp();
bool isTdustIdentTemp();

/* Identification of Temperature with Tdust: */
void setTempIdentTdust();
void unsetTempIdentTdust();
bool isTempIdentTdust();


/* Results: */
bool isCurrentResultModel(const std::string resultID);
bool isCurrentResultFunction(const std::string resultID);

const bool isConfigurationComplete();

void finalizeConfiguration();
bool isFinalizedConfiguration();

/* Usage: */
double density(double x, double y, double z);
double temperature(double x, double y, double z);
double abundance(double x, double y, double z);
double t_dust(double x, double y, double z);
double doppler(double x, double y, double z);
std::vector<double> velocity(double x, double y, double z);
std::vector<double> bmag(double x, double y, double z);
