////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef MODELREGISTRY_H
#define MODELREGISTRY_H

#include <string>
#include <set>
#include <map>
#include <vector>

#include "model.h"
#include "parameterregistry.h"

using namespace std;

class ModelRegistry
{
  ////////////////////////////////////////////////////////////////////
  //! Implementation of the Singleton pattern.
  /**
   * @name Singleton
   */
  //@{
public:
  static ModelRegistry* getInstance();
  virtual ~ModelRegistry();
protected:
  ModelRegistry();
private:
  static ModelRegistry* m_theModelRegistry;
  //@}

  // Other:
public:
  void registerModels();

  bool isRegisteredModel(const string& modelID) const;
  set<string> getModelIDs() const;
  string getModelName(const string& modelID) const;
  string getModelDesc(const string& modelID) const;
  string getModelBibref(const string& modelID) const;

  bool isRegisteredParam(const string& modelID, const string& paramID) const;
  set<string> getParamIDs(const string& modelID) const;
  string getParamType(const string& modelID, const string& paramID) const;
  string getParamName(const string& modelID, const string& paramID) const;
  string getParamDesc(const string& modelID, const string& paramID) const;
  string getParamUnit(const string& modelID, const string& paramID) const;
  int getParamDefValInt(const string& modelID, const string& paramID) const;
  double getParamDefValDouble(const string& modelID, const string& paramID) const;
  string getParamDefValString(const string& modelID, const string& paramID) const;
  vector<double> getParamEnumValues(const string& modelID, const string& paramID) const;
  unsigned getParamEnumDefIndex(const string& modelID, const string& paramID) const;
  
  Model* createModel(const string modelID) const;
  
private:
  // Model registry:
  set<string> m_modelIDs;
  map<string, ModelReg> m_modelRegs;

  void registerModel(const string& modelID);
  void addModelReg(const string& modelID, const ModelReg modelReg);
  const ModelReg& getModelReg(const string& modelID) const;

  // Parameter registries:
  map<string, ParameterRegistry> m_parameterRegistries; // Key: modelID
  const ParameterRegistry& getParameterRegistry(const string& modelID) const;
};


#endif  // MODELREGISTRY_H
