////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#include "providerregistry.h"

#include <stdexcept>
#include "iProvider.h"

using namespace std;

// Singleton pattern
ProviderRegistry* ProviderRegistry::m_theProviderRegistry = 0;

ProviderRegistry::ProviderRegistry(){}

ProviderRegistry::~ProviderRegistry(){}

ProviderRegistry* ProviderRegistry::getInstance()
{
  if (m_theProviderRegistry == 0) {
    m_theProviderRegistry = new ProviderRegistry();
  }
  return m_theProviderRegistry;
}

// Other
const void
ProviderRegistry::registerProviders()
{
	registerProvider(IDensityProvider::providerID,
	                 IDensityProvider::providerName,
	                 IDensityProvider::providerDesc,
                         IDensityProvider::providerUnit,
	                 IDensityProvider::providerType);
	registerProvider(ITemperatureProvider::providerID,
	                 ITemperatureProvider::providerName,
                         ITemperatureProvider::providerDesc,
                         ITemperatureProvider::providerUnit,
	                 ITemperatureProvider::providerType);
	registerProvider(IAbundanceProvider::providerID,
	                 IAbundanceProvider::providerName,
                         IAbundanceProvider::providerDesc,
                         IAbundanceProvider::providerUnit,
	                 IAbundanceProvider::providerType);
	registerProvider(ITdustProvider::providerID,
	                 ITdustProvider::providerName,
                         ITdustProvider::providerDesc,
                         ITdustProvider::providerUnit,
	                 ITdustProvider::providerType);
	registerProvider(IDopplerProvider::providerID,
	                 IDopplerProvider::providerName,
                         IDopplerProvider::providerDesc,
                         IDopplerProvider::providerUnit,
	                 IDopplerProvider::providerType);
	registerProvider(IVelocityProvider::providerID,
	                 IVelocityProvider::providerName,
                         IVelocityProvider::providerDesc,
                         IVelocityProvider::providerUnit,
	                 IVelocityProvider::providerType);
	registerProvider(IBmagProvider::providerID,
	                 IBmagProvider::providerName,
                         IBmagProvider::providerDesc,
                         IBmagProvider::providerUnit,
	                 IBmagProvider::providerType);
}

const void
ProviderRegistry::registerProvider(const string providerID,
                                   const string providerName,
                                   const string providerDesc,
                                   const string providerUnit,
                                   const string providerType)
{
	if (isRegisteredProvider(providerID))
	    throw invalid_argument("Multiple registration of providerID " + providerID);

	ProviderReg providerReg;
	providerReg.name = providerName;
	providerReg.description = providerDesc;
        providerReg.unit = providerUnit;
	providerReg.type = providerType;

	m_providerRegs[providerID] = providerReg;
	m_providerIDs.insert(providerID);
}

const bool
ProviderRegistry::isRegisteredProvider(const string providerID)
{
	if (m_providerRegs.count(providerID))
		return true;
	return false;
}


const set<string>
ProviderRegistry::getProviderIDs()
{
	return m_providerIDs;
}

const string
ProviderRegistry::getProviderName(const string providerID)
{
	return getProviderReg(providerID)->name;
}

const string
ProviderRegistry::getProviderDesc(const string providerID)
{
        return getProviderReg(providerID)->description;
}

const string
ProviderRegistry::getProviderUnit(const string providerID)
{
        return getProviderReg(providerID)->unit;
}

const string
ProviderRegistry::getProviderType(const string providerID)
{
	return getProviderReg(providerID)->type;

}

const ProviderReg*
ProviderRegistry::getProviderReg(const string providerID)
{
  if (!isRegisteredProvider(providerID))
    throw invalid_argument("Invalid providerID " + providerID);
  return &m_providerRegs[providerID];
}
