////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef PROVIDERREGISTRY_H
#define PROVIDERREGISTRY_H

#include <string>
#include <set>
#include <map>

using namespace std;

struct ProviderReg {
	string name;
	string description;
        string unit;
	string type;
};

class ProviderRegistry {
	// Singleton pattern:
public:
	static ProviderRegistry* getInstance();
	virtual ~ProviderRegistry();
protected:
	ProviderRegistry();
private:
	static ProviderRegistry* m_theProviderRegistry;

// Other:
public:
	const void registerProviders();

	const bool isRegisteredProvider(const string providerID);
	const set<string> getProviderIDs();
	const string getProviderName(const string providerID);
	const string getProviderDesc(const string providerID);
        const string getProviderUnit(const string providerID);
	const string getProviderType(const string providerID);

private:
	set<string> m_providerIDs;
	map<string, ProviderReg> m_providerRegs;

	const void registerProvider(const string providerID,
				    const string providerName,
                                    const string providerDesc,
                                    const string providerUnit,
				    const string providerType);

	const ProviderReg* getProviderReg(const string providerID);
};

#endif // PROVIDERREGISTRY_H
