////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef UTILITIES_H
#define UTILITIES_H


#include <string>
#include <vector>

using namespace std;

////////////////////////////////////////////////////////////////////
//
// String handling routines inspired by Python:
//
// split:
//   Return a vector of the words in the string s, using sep as the
//   delimiter string. If sep is not specified, any whitespace string
//   is a separator.
vector<string> split(const string& s);
vector<string> split(const string& s, const string& sep);

// lstrip:
//   Return a copy of the string s with leading whitespace removed.
//   If chars is given, remove characters in chars instead.
string lstrip(string s);
string lstrip(string s, const string& chars);

// rstrip:
//   Return a copy of the string s with trailing whitespace removed.
//   If chars is given, remove characters in chars instead.
string rstrip(string s);
string rstrip(string s, const string& chars);

// strip:
//   Return a copy of the string s with leading and trailing whitespace removed.
//   If chars is given, remove characters in chars instead.
string strip(string s);
string strip(string s, const string& chars);

// upper:
//   Return a copy of the string s converted to uppercase.
string upper(string s);

// lower:
//   Return a copy of the string s converted to lowercase.
string lower(string s);

////////////////////////////////////////////////////////////////////
//
// Other useful routines:
//
// heapsort:
//   Sort an array into ascending numerical order and rearrange
//   a second array accordingly.
void
heapsort(vector<double>& arr, vector<int>& arr2);



#endif // UTILITIES_H
