import modellib
import math

def evalScalar(filename):
  outfile = file(filename, 'w')
  for x in range(-100, 101):
    for z in range(-100, 101):
      line = "%f %f %f\n" % (x, z, modellib.density(x, 0, z) )
      outfile.write(line)
    outfile.write("\n")

def evalVector(filename, plane="xz"):
  outfile = file(filename, 'w')
  if plane == "xz":
    for x in range(-100, 101, 10):
      for z in range(-100, 101, 10):
        val = modellib.bmag(x, 0, z)
        line = "%f %f %f %f\n" % (x, z, val[0], val[2] )
        outfile.write(line)
      outfile.write("\n")
  if plane == "xy":
    for x in range(-100, 101, 10):
      for y in range(-100, 101, 10):
        val = modellib.bmag(x, y, 0)
        line = "%f %f %f %f\n" % (x, y, val[0], val[1] )
        outfile.write(line)
      outfile.write("\n")

# Scalar Functions:

# scalarConst
modellib.setFunction('density', 'scalarConst')
modellib.setFunctionParamDouble('density', 'val', 1.) 
modellib.finalizeConfiguration()
evalScalar("scalarConst.dat")

# scalarPowerR
modellib.setFunction('density', 'scalarPowerR')
modellib.setFunctionParamDouble('density', 'factor', 1.) 
modellib.setFunctionParamDouble('density', 'exponent', -.5)
modellib.setFunctionParamDouble('density', 'offset', 1.) 
modellib.setFunctionParamDouble('density', 'lowerR', 10.) 
modellib.finalizeConfiguration()
evalScalar("scalarPowerR.dat")

# scalarPowerRZ
modellib.setFunction('density', 'scalarPowerRZ')
modellib.setFunctionParamDouble('density', 'factR', 1.) 
modellib.setFunctionParamDouble('density', 'expR', -.5)
modellib.setFunctionParamDouble('density', 'offsetR', 0.) 
modellib.setFunctionParamDouble('density', 'lowerR', 10.) 
modellib.setFunctionParamDouble('density', 'factZ', 1.) 
modellib.setFunctionParamDouble('density', 'expZ', -2)
modellib.setFunctionParamDouble('density', 'offsetZ', 0.) 
modellib.setFunctionParamDouble('density', 'lowerZ', 10.) 
modellib.finalizeConfiguration()
evalScalar("scalarPowerRZ.dat")

# scalarPowerRExpZ
modellib.setFunction('density', 'scalarPowerRExpZ')
modellib.setFunctionParamDouble('density', 'factR', 1.) 
modellib.setFunctionParamDouble('density', 'expR', -.5)
modellib.setFunctionParamDouble('density', 'offsetR', 0.) 
modellib.setFunctionParamDouble('density', 'lowerR', 10.) 
modellib.setFunctionParamDouble('density', 'factZ', 1.) 
modellib.setFunctionParamDouble('density', 'scaleZ', 10.)
modellib.setFunctionParamDouble('density', 'offsetZ', 0.) 
modellib.finalizeConfiguration()
evalScalar("scalarPowerRExpZ.dat")

# scalarPowerRTheta
modellib.setFunction('density', 'scalarPowerRTheta')
modellib.setFunctionParamDouble('density', 'factR', 1.) 
modellib.setFunctionParamDouble('density', 'expR', -.5)
modellib.setFunctionParamDouble('density', 'offsetR', 0.) 
modellib.setFunctionParamDouble('density', 'lowerR', 10.) 
modellib.setFunctionParamDouble('density', 'factTheta', 1.) 
modellib.setFunctionParamDouble('density', 'expTheta', -2.) 
modellib.setFunctionParamDouble('density', 'offsetTheta', 0.) 
modellib.setFunctionParamDouble('density', 'lowerTheta', .26) 
modellib.finalizeConfiguration()
evalScalar("scalarPowerRTheta.dat")


# vectorConstR
modellib.setFunction('bmag', 'vectorConstR')
modellib.setFunctionParamDouble('bmag', 'val', 5.) 
modellib.finalizeConfiguration()
evalVector("vectorConstRxz.dat", "xz")
evalVector("vectorConstRxy.dat", "xy")

# vectorConstXYZ
modellib.setFunction('bmag', 'vectorConstXYZ')
modellib.setFunctionParamDouble('bmag', 'valX', 5.) 
modellib.setFunctionParamDouble('bmag', 'valY', 5.) 
modellib.setFunctionParamDouble('bmag', 'valZ', -5.) 
modellib.finalizeConfiguration()
evalVector("vectorConstXYZxz.dat", "xz")
evalVector("vectorConstXYZxy.dat", "xy")

# vectorRadialPowerR
modellib.setFunction('bmag', 'vectorRadialPowerR')
modellib.setFunctionParamDouble('bmag', 'factor', 200.) 
modellib.setFunctionParamDouble('bmag', 'exponent', -1.) 
modellib.setFunctionParamDouble('bmag', 'offset', 0.) 
modellib.setFunctionParamDouble('bmag', 'lowerR', 40.) 
modellib.finalizeConfiguration()
evalVector("vectorRadialPowerRxz.dat", "xz")
evalVector("vectorRadialPowerRxy.dat", "xy")

# vectorToroidalPowerR
modellib.setFunction('bmag', 'vectorToroidalPowerR')
modellib.setFunctionParamDouble('bmag', 'factor', 200.) 
modellib.setFunctionParamDouble('bmag', 'exponent', -1.) 
modellib.setFunctionParamDouble('bmag', 'offset', 0.) 
modellib.setFunctionParamDouble('bmag', 'lowerRho', 40.) 
modellib.finalizeConfiguration()
evalVector("vectorToroidalPowerRxz.dat", "xz")
evalVector("vectorToroidalPowerRxy.dat", "xy")

# vectorRadialPowerRTheta
modellib.setFunction('bmag', 'vectorRadialPowerRTheta')
modellib.setFunctionParamDouble('bmag', 'factR', 200.) 
modellib.setFunctionParamDouble('bmag', 'expR', -1.)
modellib.setFunctionParamDouble('bmag', 'offsetR', 0.) 
modellib.setFunctionParamDouble('bmag', 'lowerR', 40.) 
modellib.setFunctionParamDouble('bmag', 'factTheta', 1.) 
modellib.setFunctionParamDouble('bmag', 'expTheta', -1.) 
modellib.setFunctionParamDouble('bmag', 'offsetTheta', 0.) 
modellib.setFunctionParamDouble('bmag', 'lowerTheta', .26) 
modellib.finalizeConfiguration()
evalVector("vectorRadialPowerRThetaxz.dat", "xz")
evalVector("vectorRadialPowerRThetaxy.dat", "xy")

# vectorDipole
modellib.setFunction('bmag', 'vectorDipole')
modellib.setFunctionParamDouble('bmag', 'mDipole', 1.e6) 
modellib.setFunctionParamDouble('bmag', 'lowerR', 40.) 
modellib.finalizeConfiguration()
evalVector("vectorDipolexz.dat", "xz")
evalVector("vectorDipolexy.dat", "xy")
