import sys
import modellib

def usage():
    print "%s inputfile outputfile" % sys.argv[0]
    print
    print "Test Model with parameters as specified in inputfile and write results to outputfile"
    print 
    print "The Model is setup with modelID and Parameters specified in"
    print "inputfile. Points where all available Results are calculated are"
    print "specified in inputfile as well. Results are written to outputfile."
    print 



# Parse argv:
if len(sys.argv) != 3:
    usage()
    sys.exit()
    
infilename = sys.argv[1]
outfilename = sys.argv[2]


# Things to be specified in the input file:
modelID = None
params = {}    # Key: paramID; Value: value of Parameter
points = []


# Parse inputfile:
infile = open(infilename)
for line in infile.readlines():
    # Strip Comments:
    if '#' in line:
        line = line.split('#')[0]
    
    line = line.strip()
    if line:
        if '=' in line:
            # Line is keyword = value:
            words = line.split('=')
            keyword = words[0].strip()
            value = words[1].strip()
            
            if keyword == "modelID":
                # modelID:
                modelID = value
            else:
                # Parameter:
                try:
                    # Dirty trick:
                    # Try to convert to number
                    params[keyword] = float(value)
                except:
                    # If it fails, leave it a string
                    params[keyword] = val
        else:
            # Line not keyword = value
            # Then it is a point:
            # x y z
            words = line.split()
            if len(words) != 3:
                raise Exception("Invalid coordinate specification in file %s" % infilename)
            point = []
            for word in words:
                point.append(float(word.strip()))
                
            points.append(point)
            
# Write infos to output file:
outfile = open(outfilename, 'w')
outfile.write("# modelID = %s\n" % modelID)

outfile.write("#\n")
outfile.write("# Parameters:\n")
for paramID in params.keys():
    outfile.write("# %s = %s\n" % (paramID, str(params[paramID])))
        
outfile.write("#\n")
outfile.write("# Results:\n")
            
# Now do the test:            
try:
    # Set current Model:
    modellib.setCurrentModel(modelID)
    
    # Set Parameters:
    paramIDs = params.keys()
    
    # Some checks first:
    # Check that all required Parameters are defined:
    for modelParamID in modellib.getParamIDs(modelID):
        if not modelParamID in paramIDs:
            raise Exception("Parameter %s required but not set" % modelParamID)
    # Check that all defined Parameters are required:
    for paramID in paramIDs:
        if not paramID in modellib.getParamIDs(modelID):
            raise Exception("Parameter %s set but not required" % paramID)
            
    # Now set the Parameters:
    setParam = {"double": modellib.setParamDouble,
                "int"   : modellib.setParamInt,
                "string": modellib.setParamString }
    for paramID in paramIDs:
        datatype = modellib.getParamType(modelID, paramID)
        value = params[paramID]
        try:
            setParam[datatype](paramID, value)
        except Exception, e:
            msg = "Setting Parameter %s: " % paramID
            msg += e.message
            raise Exception(msg)
            
    # Setup finished:
    modellib.finalizeConfiguration()
    
    # Determine which Results are available for the Model:
    resultIDs = list(modellib.getModelResultIDs(modelID))
    resultIDs.sort()

    # Write header:
    header = "# x y z"
    for resultID in resultIDs:
        header += " %s" % modellib.getResultName(resultID)
        if modellib.getResultType(resultID) == "vector":
            header += " (vector)"
    outfile.write(header + "\n")
    
    # Dispatch function:
    value = {'abundance':   modellib.abundance,
             'bmag':        modellib.bmag,
             'density':     modellib.density,
             'doppler':     modellib.doppler,
             'tdust':       modellib.t_dust,
             'temperature': modellib.temperature,
             'velocity':    modellib.velocity }
    
    # Print results:
    for point in points:
        x, y, z = point
        line = "%e %e %e" % (x, y, z)
        for resultID in resultIDs:
            val = value[resultID](x, y, z) 
            if modellib.getResultType(resultID) == "scalar":
                line += " %e" % val
            else:
                valx, valy, valz = val
                line += " %e %e %e" % (valx, valy, valz)
        outfile.write(line + "\n")
    

except Exception, e:
    print e




