#include <stdlib.h>
#include <stdio.h>

#include "icModellib.h"

int main()
{
  int iModel, nModels, iFunction, nFunctions, iParam, nParams, iResult, nResults;
  char **modelIDs, **functionIDs, **paramIDs, **resultIDs;
  char *modelID, *functionID, *paramID, *resultID, *paramType;

  double dens;

  unsigned nValues;
  double* values;
  int iValue;

  nModels = getNumModels();
  modelIDs = getModelIDs();
  for (iModel = 0; iModel < nModels; iModel++) {
    modelID = modelIDs[iModel];
    printf("Model:\n");
    printf("\t%s:%s\t%s\t%s\n", modelID,
                            getModelName(modelID),
                            getModelDesc(modelID),
                            getModelBibref(modelID));

    printf("Parameters:\n");
    nParams = getNumParams(modelID);
    paramIDs = getParamIDs(modelID);
    for (iParam=0; iParam < nParams; iParam++) {
      paramID = paramIDs[iParam];
      printf("\t%s:", paramID);
      printf("\t%s", getParamType(modelID, paramID));
      printf("\t%s", getParamName(modelID, paramID));
      printf("\t%s", getParamDesc(modelID, paramID));

      if (strcmp(getParamType(modelID, paramID), "double") == 0) {
          printf("\t%s", getParamUnit(modelID, paramID));
          printf("\t%g\n", getParamDefValDouble(modelID, paramID));
      }
      else if (strcmp(getParamType(modelID, paramID), "int") == 0)
        printf("\t%d\n", getParamDefValInt(modelID, paramID));
      else if (strcmp(getParamType(modelID, paramID), "string") == 0)
        printf("\t%s\n", getParamDefValString(modelID, paramID));
      else if (strcmp(getParamType(modelID, paramID), "enum") == 0) {
          printf("\t%s", getParamUnit(modelID, paramID));
          printf("\t%d", getParamEnumDefIndex(modelID, paramID));
          printf("\t(");
          nValues = getNumParamEnumValues(modelID, paramID);
          values = getParamEnumValues(modelID, paramID);
          for (iValue=0; iValue<nValues; iValue++)
              printf("%g, ", values[iValue]);
          printf(")\n");
      }
      else
        printf("\t** Invalid type '%s'**\n", getParamType(modelID, paramID));
    }

    printf("Results:\n");
    nResults = getNumModelResults(modelID);
    resultIDs = getModelResultIDs(modelID);
    for (iResult=0; iResult < nResults; iResult++) {
      resultID = resultIDs[iResult];
      printf("\t%s:", resultID);
      printf("\t%s", getResultName(resultID));
      printf("\t%s", getResultDesc(resultID));
      printf("\t%s", getResultUnit(resultID));
      printf("\t%s\n", getResultType(resultID));
    }
    printf("-----\n");
  }

  nFunctions = getNumFunctions();
  functionIDs = getFunctionIDs();
  for (iFunction = 0; iFunction < nFunctions; iFunction++) {
    functionID = functionIDs[iFunction];
    printf("Function:\n");
    printf("\t%s:%s\t%s\t%s\n", functionID, getFunctionName(functionID), getFunctionDesc(functionID),  getFunctionType(functionID));

    printf("Parameters:\n");
    nParams = getNumFunctionParams(functionID);
    paramIDs = getFunctionParamIDs(functionID);
    for (iParam=0; iParam < nParams; iParam++) {
      paramID = paramIDs[iParam];
      printf("\t%s:", paramID);
      printf("\t%s", getFunctionParamType(functionID, paramID));
      printf("\t%s", getFunctionParamName(functionID, paramID));
      printf("\t%s", getFunctionParamDesc(functionID, paramID));

      if (strcmp(getFunctionParamType(functionID, paramID), "double") == 0)
          printf("\t%g\n", getFunctionParamDefValDouble(functionID, paramID));
      else if (strcmp(getFunctionParamType(functionID, paramID), "int") == 0)
        printf("\t%d\n", getFunctionParamDefValInt(functionID, paramID));
      else if (strcmp(getFunctionParamType(functionID, paramID), "string") == 0)
        printf("\t%s\n", getFunctionParamDefValString(functionID, paramID));
      else if (strcmp(getFunctionParamType(functionID, paramID), "enum") == 0) {
          printf("\t%d", getFunctionParamEnumDefIndex(functionID, paramID));
          printf("\t(");
          nValues = getNumFunctionParamEnumValues(functionID, paramID);
          values = getFunctionParamEnumValues(functionID, paramID);
          for (iValue=0; iValue<nValues; iValue++)
              printf("%g, ", values[iValue]);
          printf(")\n");
      }
      else
        printf("\t** Invalid type '%s'**\n", getFunctionParamType(functionID, paramID));
    }
    printf("-----\n");
  }

  printf("Configuration:\n");
  setCurrentModel("BonnorEbert56");
  setParamDouble("T", 100.);
  setParamDouble("rhoc", 1.e5);

  nResults = getNumModelResults(modelID);
  resultIDs = getModelResultIDs(modelID);
  for (iResult=0; iResult < nResults; iResult++) {
      resultID = resultIDs[iResult];
      if ( ! isResultModel(resultID, "BonnorEbert56") ) {
        if (strcmp(getResultType(resultID), "scalar") == 0) {
          setFunction(resultID, "scalarConst");
        } else {
          setFunction(resultID, "vectorConstR");
        }
      }
  }

  finalizeConfiguration();

  printf("Usage:\n");
  dens = modellibDensity(3.e14, 3.e14, 3.e14);
  printf("Density BonnorEbert56: %.3e\n", dens);

  return 0;
}
