#include <iostream>
#include <string>
#include <set>
#include <vector>
#include <exception>

#include "icppModellib.h"

using namespace std;

int main()
{
  try {
    set<string> modelIDs = getModelIDs();
    set<string>::iterator itModel;
    for (itModel = modelIDs.begin(); itModel != modelIDs.end(); ++itModel) {
      string modelID = *itModel;

      cout << "Model:" << endl;
      cout << "\t" << modelID << ":\t";
      cout << getModelName(modelID) << "\t";
      cout << getModelDesc(modelID) << "\t";
      cout << getModelBibref(modelID) << endl;

      cout << "Parameters:" << endl;
      set<string> paramIDs = getParamIDs(modelID);
      set<string>::iterator itParam;
      for (itParam = paramIDs.begin(); itParam != paramIDs.end(); ++itParam) {
        string paramID = *itParam;
        cout << "\t" << paramID << ":\t";
        cout << getParamType(modelID, paramID) << "\t";
        cout << getParamName(modelID, paramID) << "\t";
        cout << getParamDesc(modelID, paramID) << "\t";
        if (getParamType(modelID, paramID) == "double")
          cout << getParamUnit(modelID, paramID) << "\t"
               << getParamDefValDouble(modelID, paramID) << endl;
        else if (getParamType(modelID, paramID) == "int")
          cout << getParamDefValInt(modelID, paramID) << endl;
        else if (getParamType(modelID, paramID) == "string")
          cout << getParamDefValString(modelID, paramID) << endl;
        else if (getParamType(modelID, paramID) == "enum") {
          cout << getParamUnit(modelID, paramID) << "\t"
               << getParamEnumDefIndex(modelID, paramID) << "\t(";
          vector<double> values = getParamEnumValues(modelID, paramID);
          for (int index=0; index < values.size(); index++)
            cout << values.at(index) << ", ";
          cout << ")" << endl;
        }
        else
          cout << "** Invalid type **" << endl;
      }

      cout << "Results:" << endl;
      set<string> resultIDs = getModelResultIDs(modelID);
      set<string>::iterator itResult;
      for (itResult = resultIDs.begin(); itResult != resultIDs.end(); ++itResult) {
        string resultID = *itResult;
        cout << "\t" << resultID << ":\t";
        cout << getResultName(resultID) << "\t";
        cout << getResultDesc(resultID) << "\t";
        cout << getResultUnit(resultID) << "\t";
        cout << getResultType(resultID) << endl;
      }
      cout << "----" << endl;
    }

    set<string> functionIDs = getFunctionIDs();
    set<string>::iterator itFunction;
    for (itFunction = functionIDs.begin(); itFunction != functionIDs.end(); ++itFunction) {
      string functionID = *itFunction;

      cout << "Function:" << endl;
      cout << "\t" << functionID << ":\t";
      cout << getFunctionName(functionID) << "\t";
      cout << getFunctionDesc(functionID) << "\t";
      cout << getFunctionType(functionID) << endl;

      cout << "Parameters:" << endl;
      set<string> paramIDs = getFunctionParamIDs(functionID);
      set<string>::iterator itParam;
      for (itParam = paramIDs.begin(); itParam != paramIDs.end(); ++itParam) {
        string paramID = *itParam;
        cout << "\t" << paramID << ":\t";
        cout << getFunctionParamType(functionID, paramID) << "\t";
        cout << getFunctionParamName(functionID, paramID) << "\t";
        cout << getFunctionParamDesc(functionID, paramID) << "\t";
        if (getFunctionParamType(functionID, paramID) == "double")
          cout << getFunctionParamDefValDouble(functionID, paramID) << endl;
        else if (getFunctionParamType(functionID, paramID) == "int")
          cout << getFunctionParamDefValInt(functionID, paramID) << endl;
        else if (getFunctionParamType(functionID, paramID) == "string")
          cout << getFunctionParamDefValString(functionID, paramID) << endl;
        else if (getFunctionParamType(functionID, paramID) == "enum") {
          cout << getFunctionParamEnumDefIndex(functionID, paramID) << "\t(";
          vector<double> values = getFunctionParamEnumValues(functionID, paramID);
          for (int index=0; index < values.size(); index++)
            cout << values.at(index) << ", ";
          cout << ")" << endl;
        }
        else
          cout << "** Invalid type **" << endl;
      }
      cout << "----" << endl;
    }

    cout << "Configuration:" << endl;
    string modelID = "BonnorEbert56";
    setCurrentModel(modelID);
    setParamDouble("T", 100.);
    setParamDouble("rhoc", 1.e5);

    set<string> resultIDs = getResultIDs();
    set<string>::iterator itResult;
    for (itResult = resultIDs.begin(); itResult != resultIDs.end(); ++itResult) {
      string resultID = *itResult;
      if ( ! isResultModel(resultID, modelID) ) {
        if ( getResultType(resultID) == "scalar" ) {
          setFunction(resultID, "scalarConst");
        } else {
          setFunction(resultID, "vectorConstR");
        }
      }
    }

    finalizeConfiguration();

    cout << "Usage:" << endl;
    double dens = density(3.e14, 3.e14, 3.e14);
    cout << "Density BonnorEbert56: " << dens << endl;

  } catch (exception &e) {
    cout << "Exception: " << e.what() << endl;
    return 1;
  } catch (string msg) {
    cout << "Error: " << msg << endl;
    return 1;
  } catch (...) {
    cout << "Unspecified error" << endl;
    return 1;
  }

  return 0;
}
