////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

// Template source file for model classes
// ADJUST mark lines that must be modified

// ADJUST: Replace all occurrences of ModelTemplate with new class name

#include <stdexcept>

// ADJUST: Include correct header file
#include "modelTemplate.h"


// ADJUST: Rename Constructor and Destructor
ModelTemplate::ModelTemplate()
{}

ModelTemplate::~ModelTemplate()
{}


// ADJUST: Specify unique modelID
string
ModelTemplate::modelID = "template";

const ModelReg
ModelTemplate::getModelReg()
{
  ModelReg modelReg;

  // ADJUST: Specify the model's name, description, and bibliographic reference
  modelReg.name = "Template model";
  modelReg.description = "Just a model template";
  modelReg.bibref = "Bibliographic reference";

  return modelReg;
}

const set<string>
ModelTemplate::getParamIDs()
{
  set<string> paramIDs;

  // ADJUST: Insert IDs for all needed parameters
  paramIDs.insert("paramDbl");
  paramIDs.insert("paramInt");
  paramIDs.insert("paramStr");
  paramIDs.insert("paramEnum");

  return paramIDs;
}

const ParamReg*
ModelTemplate::getParamReg(string paramID)
{
  // ADJUST: Create and return a new ParamReg object of the correct type
  //         for each paramID specified in getParamIDs
  if (paramID == "paramDbl")
    return new ParamRegDouble("Param Dbl",            // Name
                              "Double parameter",     // Decription
                              0.,                     // Default value
                              "");                    // Unit
  if (paramID == "paramInt")
    return new ParamRegInt("Param Int",               // Name
                           "Integer parameter",       // Description
                           0);                        // Default value
  if (paramID == "paramStr")
    return new ParamRegString("Param Str",            // Name
                              "String parameter",     // Description
                              "");                    // Default value
  if (paramID == "paramEnum") {
    int nValues = 5;
    double values[] = {0., 0.5, 1.0, 2.0, 5,};
    unsigned defIndex = 0;
    return new ParamRegEnum("Param Enum",             // Name
                            "Enum Parameter",         // Description
                            nValues,                  // Number of enum values
                            values,                   // Enum values
                            defIndex,                 // Default index
                            "");                      // Unit
  throw invalid_argument("Invalid paramID " + paramID + " for modelID " + this->modelID);
}


void
ModelTemplate::onFinalizeConfiguration()
{
  // ADJUST: Implement whatever is needed here
  m_pDbl = m_paramDouble["paramDbl"];
  m_pInt = m_paramInt["paramInt"];
  m_pStr = m_paramString["paramStr"];
  m_pEnumIndex = m_paramEnumIndex["paramEnum"];
}

// ADJUST: Remove methods that do not apply, implement the rest
// Related with IDensityProvider
const double
ModelTemplate::density(const double x, const double y, const double z)
{
  return 0;
}

// Related with ITemperatureProvider
const double
ModelTemplate::temperature(const double x, const double y, const double z)
{
  return 0;
}

// Related with ITdustProvider
const double
ModelTemplate::t_dust(const double x, const double y, const double z)
{
  return 0;
}

// Related with IAbundanceProvider
const double
ModelTemplate::abundance(const double x, const double y, const double z)
{
  return 0;
}

// Related with IDopplerProvider
const double
ModelTemplate::doppler(const double x, const double y, const double z)
{
  return 0;
}

// Related with IVelocityProvider
const void
ModelTemplate::velocity(const double x, const double y, const double z, double* v)
{
  v[0] = 0;
  v[1] = 0;
  v[2] = 0;
}

// Related with IBmagProvider
const void
ModelTemplate::bmag(const double x, const double y, const double z, double* b)
{
  b[0] = 0;
  b[1] = 0;
  b[2] = 0;
}


