////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

#include <stdexcept>
#include <math.h>

#include "modelUser.h"

string
ModelUser::modelID = "user";

const ModelReg
ModelUser::getModelReg()
{
  ModelReg modelReg;
  modelReg.name = "User model";
  modelReg.description = "User supplied model";
  modelReg.bibref = "";
  return modelReg;
}

const set<string>
ModelUser::getParamIDs()
{
  set<string> paramIDs;
  paramIDs.insert("paramDbl1");
  paramIDs.insert("paramDbl2");
  paramIDs.insert("paramDbl3");
  paramIDs.insert("paramDbl4");
  paramIDs.insert("paramDbl5");

  paramIDs.insert("paramInt1");
  paramIDs.insert("paramInt2");
  paramIDs.insert("paramInt3");
  paramIDs.insert("paramInt4");
  paramIDs.insert("paramInt5");

  paramIDs.insert("paramStr1");
  paramIDs.insert("paramStr2");
  paramIDs.insert("paramStr3");
  paramIDs.insert("paramStr4");
  paramIDs.insert("paramStr5");

  return paramIDs;
}

const ParamReg*
ModelUser::getParamReg(string paramID)
{
  if (paramID == "paramDbl1")
    return new ParamRegDouble("m_paramDbl1",
                              "Double parameter 1",
                              0.,
                              "");
  else if (paramID == "paramDbl2")
    return new ParamRegDouble("m_paramDbl2",
                              "Double parameter 2",
                              0.,
                              "");
  else if (paramID == "paramDbl3")
    return new ParamRegDouble("m_paramDbl3",
                              "Double parameter 3",
                              0.,
                              "");
  else if (paramID == "paramDbl4")
    return new ParamRegDouble("m_paramDbl4",
                              "Double parameter 4",
                              0.,
                              "");
  else if (paramID == "paramDbl5")
    return new ParamRegDouble("m_paramDbl5",
                              "Double parameter 5",
                              0.,
                              "");

  else if (paramID == "paramInt1")
    return new ParamRegInt("m_paramInt1",
                           "Integer parameter 1",
                           0);
  else if (paramID == "paramInt2")
    return new ParamRegInt("m_paramInt2",
                           "Integer parameter 2",
                           0);
  else if (paramID == "paramInt3")
    return new ParamRegInt("m_paramInt3",
                           "Integer parameter 3",
                           0);
  else if (paramID == "paramInt4")
    return new ParamRegInt("m_paramInt4",
                           "Integer parameter 4",
                           0);
  else if (paramID == "paramInt5")
    return new ParamRegInt("m_paramInt5",
                           "Integer parameter 5",
                           0);

  else if (paramID == "paramStr1")
    return new ParamRegString("m_paramStr1",
                           "String parameter 1",
                           "");
  else if (paramID == "paramStr2")
    return new ParamRegString("m_paramStr2",
                           "String parameter 2",
                           "");
  else if (paramID == "paramStr3")
    return new ParamRegString("m_paramStr3",
                           "String parameter 3",
                           "");
  else if (paramID == "paramStr4")
    return new ParamRegString("m_paramStr4",
                           "String parameter 4",
                           "");
  else if (paramID == "paramStr5")
    return new ParamRegString("m_paramStr5",
                           "String parameter 5",
                           "");

  else
    throw invalid_argument("Invalid paramID " + paramID + " for modelID " + this->modelID);
}



ModelUser::ModelUser()
{}

ModelUser::~ModelUser()
{}


void
ModelUser::onFinalizeConfiguration()
{
  m_paramDbl1 = m_paramDouble["paramDbl1"];
  m_paramDbl2 = m_paramDouble["paramDbl2"];
  m_paramDbl3 = m_paramDouble["paramDbl3"];
  m_paramDbl4 = m_paramDouble["paramDbl4"];
  m_paramDbl5 = m_paramDouble["paramDbl5"];

  m_paramInt1 = m_paramInt["paramInt1"];
  m_paramInt2 = m_paramInt["paramInt2"];
  m_paramInt3 = m_paramInt["paramInt3"];
  m_paramInt4 = m_paramInt["paramInt4"];
  m_paramInt5 = m_paramInt["paramInt5"];

  m_paramStr1 = m_paramString["paramStr1"];
  m_paramStr2 = m_paramString["paramStr2"];
  m_paramStr3 = m_paramString["paramStr3"];
  m_paramStr4 = m_paramString["paramStr4"];
  m_paramStr5 = m_paramString["paramStr5"];

}

const double
ModelUser::density(const double x, const double y, const double z)
{
  return 0;
}

const double
ModelUser::temperature(const double x, const double y, const double z)
{
  return 0;
}

const double
ModelUser::t_dust(const double x, const double y, const double z)
{
  return 0;
}

const double
ModelUser::abundance(const double x, const double y, const double z)
{
  return 0;
}

const double
ModelUser::doppler(const double x, const double y, const double z)
{
  return 0;
}

const void
ModelUser::velocity(const double x, const double y, const double z, double* v)
{
  v[0] = 0;
  v[1] = 0;
  v[2] = 0;
}

const void
ModelUser::bmag(const double x, const double y, const double z, double* b)
{
  b[0] = 0;
  b[1] = 0;
  b[2] = 0;
}


